# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DMRI_PATHS_METADATA = Metadata(
    id="6eda0635b358ac171fbbf14d968294d81a352fc1.boutiques",
    name="dmri_paths",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class DmriPathsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dmri_paths(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def dmri_paths(
    indir: str | None = None,
    outdir: str | None = None,
    dwi: InputPathType | None = None,
    grad: InputPathType | None = None,
    bval: InputPathType | None = None,
    mask: InputPathType | None = None,
    bpdir: str | None = None,
    ntr: float | None = 1,
    fmin: float | None = 0,
    basereg: InputPathType | None = None,
    basemask: InputPathType | None = None,
    roi1: InputPathType | None = None,
    roi2: InputPathType | None = None,
    roimesh1: InputPathType | None = None,
    roimesh2: InputPathType | None = None,
    roiref1: InputPathType | None = None,
    roiref2: InputPathType | None = None,
    prior: InputPathType | None = None,
    nprior: InputPathType | None = None,
    nset: float | None = None,
    lprior: InputPathType | None = None,
    lset: float | None = None,
    seg: InputPathType | None = None,
    tprior: InputPathType | None = None,
    cprior: InputPathType | None = None,
    reg: InputPathType | None = None,
    regnl: InputPathType | None = None,
    init: InputPathType | None = None,
    nb: float | None = 5000,
    ns: float | None = 5000,
    nk: float | None = 10,
    nu: float | None = 40,
    sdp: InputPathType | None = None,
    debug: bool = False,
    checkopts: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DmriPathsOutputs:
    """
    Tool for diffusion MRI path analysis.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        indir: Input subject directory (optional), specify multiple for\
            longitudinal data.
        outdir: Output directory (one per path).
        dwi: DWI volume series.
        grad: Text file of diffusion gradients.
        bval: Text file of diffusion b-values.
        mask: Mask volume.
        bpdir: BEDPOST directory.
        ntr: Max number of tracts per voxel (default 1).
        fmin: Tract volume fraction threshold (default 0).
        basereg: Base-to-DWI registration, needed for longitudinal data only\
            (.mat, as many as input directories).
        basemask: Base template mask volume.
        roi1: End ROI 1 (volume or label, one per path).
        roi2: End ROI 2 (volume or label, one per path).
        roimesh1: Mesh for end ROI 1 (for label ROIs).
        roimesh2: Mesh for end ROI 2 (for label ROIs).
        roiref1: Reference volume for end ROI 1 (for label ROIs).
        roiref2: Reference volume for end ROI 2 (for label ROIs).
        prior: Spatial path priors (negative log-likelihoods off and on the\
            path, one pair per path).
        nprior: Near-neighbor label priors (negative log-likelihood and list of\
            labels, one pair per path).
        nset: Subset of near-neighbor label priors (default all).
        lprior: Local-neighbor label priors (negative log-likelihood and list\
            of labels, one pair per path).
        lset: Subset of local-neighbor label priors (default all).
        seg: Segmentation map of test subject, specify multiple for\
            longitudinal data.
        tprior: Path tangent vector priors (negative log-likelihood, one per\
            path).
        cprior: Path curvature priors (negative log-likelihood, one per path).
        reg: DWI-to-atlas affine registration (.mat).
        regnl: DWI-to-atlas nonlinear registration (.m3z).
        init: Text file of initial control points (one per path).
        nb: Number of burn-in samples (default 5000).
        ns: Number of post-burn-in samples (default 5000).
        nk: Keep every nk-th sample (default 10).
        nu: Update proposal every nu-th sample (default 40).
        sdp: Text file with initial proposal standard deviations for control\
            point perturbations.
        debug: Turn on debugging.
        checkopts: Don't run anything, just check options and exit.
        version: Print out version and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DmriPathsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DMRI_PATHS_METADATA)
    cargs = []
    cargs.append("dmri_paths")
    if indir is not None:
        cargs.extend([
            "--indir",
            indir
        ])
    if outdir is not None:
        cargs.extend([
            "--outdir",
            outdir
        ])
    if dwi is not None:
        cargs.extend([
            "--dwi",
            execution.input_file(dwi)
        ])
    if grad is not None:
        cargs.extend([
            "--grad",
            execution.input_file(grad)
        ])
    if bval is not None:
        cargs.extend([
            "--bval",
            execution.input_file(bval)
        ])
    if mask is not None:
        cargs.extend([
            "--mask",
            execution.input_file(mask)
        ])
    if bpdir is not None:
        cargs.extend([
            "--bpdir",
            bpdir
        ])
    if ntr is not None:
        cargs.extend([
            "--ntr",
            str(ntr)
        ])
    if fmin is not None:
        cargs.extend([
            "--fmin",
            str(fmin)
        ])
    if basereg is not None:
        cargs.extend([
            "--basereg",
            execution.input_file(basereg)
        ])
    if basemask is not None:
        cargs.extend([
            "--basemask",
            execution.input_file(basemask)
        ])
    if roi1 is not None:
        cargs.extend([
            "--roi1",
            execution.input_file(roi1)
        ])
    if roi2 is not None:
        cargs.extend([
            "--roi2",
            execution.input_file(roi2)
        ])
    if roimesh1 is not None:
        cargs.extend([
            "--roimesh1",
            execution.input_file(roimesh1)
        ])
    if roimesh2 is not None:
        cargs.extend([
            "--roimesh2",
            execution.input_file(roimesh2)
        ])
    if roiref1 is not None:
        cargs.extend([
            "--roiref1",
            execution.input_file(roiref1)
        ])
    if roiref2 is not None:
        cargs.extend([
            "--roiref2",
            execution.input_file(roiref2)
        ])
    if prior is not None:
        cargs.extend([
            "--prior",
            execution.input_file(prior)
        ])
    if nprior is not None:
        cargs.extend([
            "--nprior",
            execution.input_file(nprior)
        ])
    if nset is not None:
        cargs.extend([
            "--nset",
            str(nset)
        ])
    if lprior is not None:
        cargs.extend([
            "--lprior",
            execution.input_file(lprior)
        ])
    if lset is not None:
        cargs.extend([
            "--lset",
            str(lset)
        ])
    if seg is not None:
        cargs.extend([
            "--seg",
            execution.input_file(seg)
        ])
    if tprior is not None:
        cargs.extend([
            "--tprior",
            execution.input_file(tprior)
        ])
    if cprior is not None:
        cargs.extend([
            "--cprior",
            execution.input_file(cprior)
        ])
    if reg is not None:
        cargs.extend([
            "--reg",
            execution.input_file(reg)
        ])
    if regnl is not None:
        cargs.extend([
            "--regnl",
            execution.input_file(regnl)
        ])
    if init is not None:
        cargs.extend([
            "--init",
            execution.input_file(init)
        ])
    if nb is not None:
        cargs.extend([
            "--nb",
            str(nb)
        ])
    if ns is not None:
        cargs.extend([
            "--ns",
            str(ns)
        ])
    if nk is not None:
        cargs.extend([
            "--nk",
            str(nk)
        ])
    if nu is not None:
        cargs.extend([
            "--nu",
            str(nu)
        ])
    if sdp is not None:
        cargs.extend([
            "--sdp",
            execution.input_file(sdp)
        ])
    if debug:
        cargs.append("--debug")
    if checkopts:
        cargs.append("--checkopts")
    if version:
        cargs.append("--version")
    ret = DmriPathsOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DMRI_PATHS_METADATA",
    "DmriPathsOutputs",
    "dmri_paths",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_APPLY_BIAS_METADATA = Metadata(
    id="b81c79a708e537c2c711365f270f39010a6db69c.boutiques",
    name="mri_apply_bias",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriApplyBiasOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_apply_bias(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_volume_file: OutputPathType
    """Output volume after applying bias"""


def mri_apply_bias(
    input_volume: InputPathType,
    bias_volume: InputPathType,
    output_volume: str,
    runner: Runner | None = None,
) -> MriApplyBiasOutputs:
    """
    A tool for applying a bias volume to an input volume to produce an output
    volume.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_volume: The input volume file.
        bias_volume: The bias volume file.
        output_volume: The output volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriApplyBiasOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_APPLY_BIAS_METADATA)
    cargs = []
    cargs.append("mri_apply_bias")
    cargs.append("[OPTIONS]")
    cargs.append(execution.input_file(input_volume))
    cargs.append(execution.input_file(bias_volume))
    cargs.append(output_volume)
    ret = MriApplyBiasOutputs(
        root=execution.output_file("."),
        output_volume_file=execution.output_file(output_volume),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_APPLY_BIAS_METADATA",
    "MriApplyBiasOutputs",
    "mri_apply_bias",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_MS_FITPARMS_METADATA = Metadata(
    id="b0347aaf3a59e1d2a9709a64ffe1e9cfa1594acd.boutiques",
    name="mri_ms_fitparms",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriMsFitparmsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_ms_fitparms(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    t1_map: OutputPathType
    """Compressed T1 map"""
    pd_map: OutputPathType
    """Compressed Proton Density map"""
    vol0_synthetic: OutputPathType
    """Compressed Synthetic volume for input vol_1, in motion corrected space"""
    vol0_affine: OutputPathType
    """Motion correction affine information for vol_1"""
    vol1_synthetic: OutputPathType
    """Compressed Synthetic volume for input vol_2, in motion corrected space"""
    vol1_affine: OutputPathType
    """Motion correction affine information for vol_2"""


def mri_ms_fitparms(
    volumes: list[InputPathType],
    output_dir: str,
    afi_flag: bool = False,
    ait_flag: bool = False,
    at: str | None = None,
    conform_flag: bool = False,
    correct_flag: bool = False,
    cubic_flag: bool = False,
    debug_slice_flag: bool = False,
    debug_voxel_flag: bool = False,
    dt: str | None = None,
    fa: str | None = None,
    fa_scale: float | None = None,
    faf: str | None = None,
    fsmooth: float | None = None,
    invert_flag: bool = False,
    momentum: str | None = None,
    max_t2: str | None = None,
    n_iter: float | None = None,
    nearest_flag: bool = False,
    nocompress_flag: bool = False,
    nosynth_flag: bool = False,
    residuals: str | None = None,
    smooth_sigma: float | None = None,
    scale_factor: float | None = None,
    sinc_flag: bool = False,
    transform_flag: bool = False,
    echo_time: float | None = None,
    repetition_time: float | None = None,
    trilinear_flag: bool = False,
    tukey_flag: bool = False,
    help_flag: bool = False,
    use_brain_mask_flag: bool = False,
    write_intermediate: float | None = None,
    extract_subimage: list[float] | None = None,
    window_flag: bool = False,
    runner: Runner | None = None,
) -> MriMsFitparmsOutputs:
    """
    Tool for estimating T1 and PD values from FLASH images and applying
    transformations.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        volumes: List of 3D FLASH images with different flip angles.
        output_dir: Output directory.
        afi_flag: Designate flip angle map to use with nominal value 60 degrees.
        ait_flag: Apply inverse of transform.
        at: Set acquisition time.
        conform_flag: Interpolate volume to be isotropic 1mm^3.
        correct_flag: Correct proton density map (PD) by T2* estimates.
        cubic_flag: Use cubic interpolation (NOT WORKING!).
        debug_slice_flag: Debug slice processing ???.
        debug_voxel_flag: Debug voxel processing ???.
        dt: Set time step dt ???.
        fa: Set flip angle.
        fa_scale: Set value to scale all flip angles by.
        faf: Designate flip angle map to use with specified control points.
        fsmooth: Smooth flip angle map for X iterations of soap bubble\
            smoothing.
        invert_flag: Invert volumes.
        momentum: Set momentum.
        max_t2: Set max T2*.
        n_iter: Perform estimation/motion correction X times.
        nearest_flag: Use nearest-neighbor interpolation.
        nocompress_flag: Do not compress output volumes (save as .mgh).
        nosynth_flag: Disable volume synthesis.
        residuals: Write out residuals to designated location.
        smooth_sigma: Smooth faf field with sigma=X.
        scale_factor: Scale volumes by X after reading.
        sinc_flag: Use sinc interpolation (default window width is 6).
        transform_flag: Apply transform to output volumes.
        echo_time: Set echo time (TE) in ms.
        repetition_time: Set repetition time (TR) in ms.
        trilinear_flag: Use trilinear interpolation.
        tukey_flag: Use Tukey bi-weight of residuals.
        help_flag: Display help text.
        use_brain_mask_flag: Compute a brain mask from the PD map and use it\
            for registration.
        write_intermediate: Write out intermediate results every X iterations.
        extract_subimage: Extract a subimage for each input image, specified by\
            x0 y0 z0 dx dy dz.
        window_flag: Not implemented.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriMsFitparmsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_MS_FITPARMS_METADATA)
    cargs = []
    cargs.append("mri_ms_fitparms")
    cargs.extend([execution.input_file(f) for f in volumes])
    cargs.append(output_dir)
    if afi_flag:
        cargs.append("-afi")
    if ait_flag:
        cargs.append("-ait")
    if at is not None:
        cargs.extend([
            "-at",
            at
        ])
    if conform_flag:
        cargs.append("-conform")
    if correct_flag:
        cargs.append("-correct")
    if cubic_flag:
        cargs.append("-cubic")
    if debug_slice_flag:
        cargs.append("-debug_slice")
    if debug_voxel_flag:
        cargs.append("-debug_voxel")
    if dt is not None:
        cargs.extend([
            "-dt",
            dt
        ])
    if fa is not None:
        cargs.extend([
            "-fa",
            fa
        ])
    if fa_scale is not None:
        cargs.extend([
            "-fa_scale",
            str(fa_scale)
        ])
    if faf is not None:
        cargs.extend([
            "-faf",
            faf
        ])
    if fsmooth is not None:
        cargs.extend([
            "-fsmooth",
            str(fsmooth)
        ])
    if invert_flag:
        cargs.append("-i")
    if momentum is not None:
        cargs.extend([
            "-m",
            momentum
        ])
    if max_t2 is not None:
        cargs.extend([
            "-max",
            max_t2
        ])
    if n_iter is not None:
        cargs.extend([
            "-n",
            str(n_iter)
        ])
    if nearest_flag:
        cargs.append("-nearest")
    if nocompress_flag:
        cargs.append("-nocompress")
    if nosynth_flag:
        cargs.append("-nosynth")
    if residuals is not None:
        cargs.extend([
            "-r",
            residuals
        ])
    if smooth_sigma is not None:
        cargs.extend([
            "-s",
            str(smooth_sigma)
        ])
    if scale_factor is not None:
        cargs.extend([
            "-scale",
            str(scale_factor)
        ])
    if sinc_flag:
        cargs.append("-sinc")
    if transform_flag:
        cargs.append("-t")
    if echo_time is not None:
        cargs.extend([
            "-te",
            str(echo_time)
        ])
    if repetition_time is not None:
        cargs.extend([
            "-tr",
            str(repetition_time)
        ])
    if trilinear_flag:
        cargs.append("-trilinear")
    if tukey_flag:
        cargs.append("-tukey")
    if help_flag:
        cargs.append("-u")
    if use_brain_mask_flag:
        cargs.append("-use_brain_mask")
    if write_intermediate is not None:
        cargs.extend([
            "-w",
            str(write_intermediate)
        ])
    if extract_subimage is not None:
        cargs.extend([
            "-extract",
            *map(str, extract_subimage)
        ])
    if window_flag:
        cargs.append("-window")
    ret = MriMsFitparmsOutputs(
        root=execution.output_file("."),
        t1_map=execution.output_file("T1.mgz"),
        pd_map=execution.output_file("PD.mgz"),
        vol0_synthetic=execution.output_file("vol0.mgz"),
        vol0_affine=execution.output_file("vol0.lta"),
        vol1_synthetic=execution.output_file("vol1.mgz"),
        vol1_affine=execution.output_file("vol1.lta"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_MS_FITPARMS_METADATA",
    "MriMsFitparmsOutputs",
    "mri_ms_fitparms",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TBSS_DEPROJECT_METADATA = Metadata(
    id="fff73a1ba7bde8cd6f4413cf518933120619f762.boutiques",
    name="tbss_deproject",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class TbssDeprojectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tbss_deproject(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    stats_output: OutputPathType
    """Output in stats directory."""
    fa_output: OutputPathType
    """Output in FA directory."""


def tbss_deproject(
    skeleton_space_input_image: InputPathType,
    final_space_option: int,
    index_image_flag: bool = False,
    runner: Runner | None = None,
) -> TbssDeprojectOutputs:
    """
    Tool to deproject images from skeleton space to final space.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        skeleton_space_input_image: Skeleton space input image.
        final_space_option: Final space option.
        index_image_flag: Assume that the skeleton space input image contains\
            one or more integer index numbers and ensure that those exact values\
            are deprojected without being changed by interpolations.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TbssDeprojectOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TBSS_DEPROJECT_METADATA)
    cargs = []
    cargs.append("tbss_deproject")
    cargs.append(execution.input_file(skeleton_space_input_image))
    cargs.append(str(final_space_option))
    if index_image_flag:
        cargs.append("-n")
    ret = TbssDeprojectOutputs(
        root=execution.output_file("."),
        stats_output=execution.output_file("stats/" + pathlib.Path(skeleton_space_input_image).name),
        fa_output=execution.output_file("FA/" + pathlib.Path(skeleton_space_input_image).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TBSS_DEPROJECT_METADATA",
    "TbssDeprojectOutputs",
    "tbss_deproject",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MERGE_STATS_TABLES_METADATA = Metadata(
    id="004967733986569ad699b7fcfc4e0e899cb02c90.boutiques",
    name="merge_stats_tables",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MergeStatsTablesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `merge_stats_tables(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    merged_stats_table: OutputPathType
    """The merged stats table"""


def merge_stats_tables(
    outputfile: str,
    meas: str,
    subject: str | None = None,
    input_: InputPathType | None = None,
    subjectsfile: InputPathType | None = None,
    intable: InputPathType | None = None,
    subdir: str | None = None,
    delimiter: str | None = None,
    common_segs: bool = False,
    all_segs: bool = False,
    transpose: bool = False,
    skip: bool = False,
    debug: bool = False,
    runner: Runner | None = None,
) -> MergeStatsTablesOutputs:
    """
    Merges a set of stats table files into a single stats table where each line is a
    subject and each column is a segmentation or parcellation.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        outputfile: The output table file.
        meas: Measure to write in output table.
        subject: Specify a single subject name.
        input_: Specify a single input stat file.
        subjectsfile: Name of the file which has the list of subjects (one\
            subject per line).
        intable: Use `fname` as input (REQUIRED when passing subject ids).
        subdir: Use `subdir` instead of default "stats/" when passing subject\
            ids.
        delimiter: Delimiter between measures in the table. Options are 'tab',\
            'space', 'comma', and 'semicolon'. Default is 'space'.
        common_segs: Output only the common segmentations of all the statsfiles\
            given.
        all_segs: Output all the segmentations of the statsfiles given.
        transpose: Transpose the table (default is subjects in rows and\
            segmentations in cols).
        skip: If a subject does not have a stats file, skip it instead of\
            exiting.
        debug: Increase verbosity for debugging purposes.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MergeStatsTablesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MERGE_STATS_TABLES_METADATA)
    cargs = []
    cargs.append("merge_stats_tables")
    if subject is not None:
        cargs.extend([
            "-s",
            subject
        ])
    if input_ is not None:
        cargs.extend([
            "-i",
            execution.input_file(input_)
        ])
    cargs.extend([
        "-t",
        outputfile
    ])
    cargs.extend([
        "-m",
        meas
    ])
    if subjectsfile is not None:
        cargs.extend([
            "--subjectsfile",
            execution.input_file(subjectsfile)
        ])
    if intable is not None:
        cargs.extend([
            "--intable",
            execution.input_file(intable)
        ])
    if subdir is not None:
        cargs.extend([
            "--subdir",
            subdir
        ])
    if delimiter is not None:
        cargs.extend([
            "-d",
            delimiter
        ])
    if common_segs:
        cargs.append("--common-segs")
    if all_segs:
        cargs.append("--all-segs")
    cargs.append("[SEGIDS_FROM_FILE]")
    cargs.append("[SEGNO]")
    cargs.append("[NO_SEGNO_FLAG]")
    if transpose:
        cargs.append("--transpose")
    if skip:
        cargs.append("--skip")
    if debug:
        cargs.append("-v")
    ret = MergeStatsTablesOutputs(
        root=execution.output_file("."),
        merged_stats_table=execution.output_file(outputfile),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MERGE_STATS_TABLES_METADATA",
    "MergeStatsTablesOutputs",
    "merge_stats_tables",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BIANCA_METADATA = Metadata(
    id="faece12893f9764ba2ed673c9b5a6791f2ed9a43.boutiques",
    name="bianca",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class BiancaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `bianca(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    base_output: OutputPathType | None
    """Base output file generated by BIANCA"""


def bianca(
    master_file: InputPathType,
    label_feature_num: float,
    brain_mask_feature_num: float,
    query_subject_num: float,
    feature_subset: str | None = None,
    mat_feature_num: float | None = None,
    spatial_weight: float | None = 1,
    patch_sizes: str | None = None,
    patch_3d: bool = False,
    select_pts: str | None = "any",
    training_pts: str | None = None,
    non_les_pts: str | None = None,
    save_classifier_data: str | None = None,
    verbose_flag: bool = False,
    out_name: str | None = "output_bianca",
    runner: Runner | None = None,
) -> BiancaOutputs:
    """
    BIANCA: Brain Intensity AbNormality Classification Algorithm.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        master_file: Name of the master file.
        label_feature_num: Column number (in the master file) of the manual\
            masks (or any placeholder name for query subjects).
        brain_mask_feature_num: Column number (in the master file) of images to\
            derive non-zero mask from.
        query_subject_num: Row number of query subject (in masterlistfile).
        feature_subset: Set of column numbers (comma separated and no spaces)\
            for features/images to use (default: use all available modalities as\
            intensity features). The image used to derive non-zero mask from must\
            be part of the features subset.
        mat_feature_num: Column number of matrix files (in masterlistfile).\
            Needed to extract spatial features (MNI coordinates).
        spatial_weight: Weighting for spatial coordinates (default = 1, i.e.,\
            variance-normalized MNI coordinates). Requires --matfeaturenum to be\
            specified.
        patch_sizes: List of patch sizes for local averaging.
        patch_3d: Use 3D patches (default is 2D).
        select_pts: "any" (default) or "surround" or "noborder".
        training_pts: Number (max) of (lesion) points to use (per training\
            subject) or "equalpoints" to select all lesion points and equal number\
            of non-lesion points.
        non_les_pts: Number (max) of non-lesion points to use. If not specified\
            will be set to the same amount of lesion points.
        save_classifier_data: Save training data to file.
        verbose_flag: Use verbose mode.
        out_name: Specify (base) output name of files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BiancaOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BIANCA_METADATA)
    cargs = []
    cargs.append("bianca")
    cargs.append("--singlefile=" + execution.input_file(master_file))
    cargs.append("--labelfeaturenum=" + str(label_feature_num))
    cargs.append("--brainmaskfeaturenum=" + str(brain_mask_feature_num))
    cargs.append("--querysubjectnum=" + str(query_subject_num))
    if feature_subset is not None:
        cargs.extend([
            "--featuresubset",
            feature_subset
        ])
    if mat_feature_num is not None:
        cargs.extend([
            "--matfeaturenum",
            str(mat_feature_num)
        ])
    if spatial_weight is not None:
        cargs.extend([
            "--spatialweight",
            str(spatial_weight)
        ])
    if patch_sizes is not None:
        cargs.extend([
            "--patchsizes",
            patch_sizes
        ])
    if patch_3d:
        cargs.append("--patch3D")
    if select_pts is not None:
        cargs.extend([
            "--selectpts",
            select_pts
        ])
    if training_pts is not None:
        cargs.extend([
            "--trainingpts",
            training_pts
        ])
    if non_les_pts is not None:
        cargs.extend([
            "--nonlespts",
            non_les_pts
        ])
    if save_classifier_data is not None:
        cargs.extend([
            "--saveclassifierdata",
            save_classifier_data
        ])
    if verbose_flag:
        cargs.append("-v")
    if out_name is not None:
        cargs.extend([
            "-o",
            out_name
        ])
    ret = BiancaOutputs(
        root=execution.output_file("."),
        base_output=execution.output_file(out_name + "_bianca") if (out_name is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BIANCA_METADATA",
    "BiancaOutputs",
    "bianca",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FSL_ANAT_METADATA = Metadata(
    id="1a5aed80642f92ca7305f50c5660ab9ad9f80c7e.boutiques",
    name="fsl_anat",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class FslAnatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fsl_anat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_anat_dir: OutputPathType | None
    """Output anatomical directory"""


def fsl_anat(
    structural_image: InputPathType | None = None,
    existing_anat_dir: str | None = None,
    output_dir: str | None = None,
    clobber_flag: bool = False,
    strongbias_flag: bool = False,
    weakbias_flag: bool = False,
    noreorient_flag: bool = False,
    nocrop_flag: bool = False,
    nobias_flag: bool = False,
    noreg_flag: bool = False,
    nononlinreg_flag: bool = False,
    noseg_flag: bool = False,
    nosubcortseg_flag: bool = False,
    bias_smoothing: float | None = None,
    image_type: str | None = None,
    nosearch_flag: bool = False,
    bet_f_param: float | None = None,
    nocleanup_flag: bool = False,
    runner: Runner | None = None,
) -> FslAnatOutputs:
    """
    A wrapper for FSL tools to process anatomical scans.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        structural_image: Filename of input image (for one image only).
        existing_anat_dir: Directory name for existing .anat directory where\
            this script will be run in place.
        output_dir: Basename of directory for output (default is input image\
            basename followed by .anat).
        clobber_flag: If .anat directory exists (as specified by -o or default\
            from -i) then delete it and make a new one.
        strongbias_flag: Used for images with very strong bias fields.
        weakbias_flag: Used for images with smoother, more typical, bias fields\
            (default setting).
        noreorient_flag: Turn off step that does reorientation to standard\
            (fslreorient2std).
        nocrop_flag: Turn off step that does automated cropping (robustfov).
        nobias_flag: Turn off steps that do bias field correction (via FAST).
        noreg_flag: Turn off steps that do registration to standard (FLIRT and\
            FNIRT).
        nononlinreg_flag: Turn off step that does non-linear registration\
            (FNIRT).
        noseg_flag: Turn off step that does tissue-type segmentation (FAST).
        nosubcortseg_flag: Turn off step that does sub-cortical segmentation\
            (FIRST).
        bias_smoothing: Specify the value for bias field smoothing (the -l\
            option in FAST).
        image_type: Specify the type of image (choose one of T1 T2 PD - default\
            is T1).
        nosearch_flag: Specify that linear registration uses the -nosearch\
            option (FLIRT).
        bet_f_param: Specify f parameter for BET (only used if not running\
            non-linear reg and also wanting brain extraction done).
        nocleanup_flag: Do not remove intermediate files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FslAnatOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FSL_ANAT_METADATA)
    cargs = []
    cargs.append("fsl_anat")
    if structural_image is not None:
        cargs.extend([
            "-i",
            execution.input_file(structural_image)
        ])
    if existing_anat_dir is not None:
        cargs.extend([
            "-d",
            existing_anat_dir
        ])
    if output_dir is not None:
        cargs.extend([
            "-o",
            output_dir
        ])
    if clobber_flag:
        cargs.append("--clobber")
    if strongbias_flag:
        cargs.append("--strongbias")
    if weakbias_flag:
        cargs.append("--weakbias")
    if noreorient_flag:
        cargs.append("--noreorient")
    if nocrop_flag:
        cargs.append("--nocrop")
    if nobias_flag:
        cargs.append("--nobias")
    if noreg_flag:
        cargs.append("--noreg")
    if nononlinreg_flag:
        cargs.append("--nononlinreg")
    if noseg_flag:
        cargs.append("--noseg")
    if nosubcortseg_flag:
        cargs.append("--nosubcortseg")
    if bias_smoothing is not None:
        cargs.extend([
            "-s",
            str(bias_smoothing)
        ])
    if image_type is not None:
        cargs.extend([
            "-t",
            image_type
        ])
    if nosearch_flag:
        cargs.append("--nosearch")
    if bet_f_param is not None:
        cargs.extend([
            "--betfparam",
            str(bet_f_param)
        ])
    if nocleanup_flag:
        cargs.append("--nocleanup")
    ret = FslAnatOutputs(
        root=execution.output_file("."),
        output_anat_dir=execution.output_file(output_dir + ".anat") if (output_dir is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FSL_ANAT_METADATA",
    "FslAnatOutputs",
    "fsl_anat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FAT_MVM_PREP_METADATA = Metadata(
    id="90f6bc657de16af423e425320b0789c1d1f1d2eb.boutiques",
    name="fat_mvm_prep",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class FatMvmPrepOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fat_mvm_prep(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    mvmtbl: OutputPathType
    """Output tabular text file for 3dMVM."""
    mvmprep_log: OutputPathType
    """Log file detailing subject matching and ROI list."""


def fat_mvm_prep(
    prefix: str,
    csv_file: InputPathType,
    matrix_files: str | None = None,
    list_match: InputPathType | None = None,
    unionize_rois: bool = False,
    na_warn_off: bool = False,
    extern_labels_no: bool = False,
    runner: Runner | None = None,
) -> FatMvmPrepOutputs:
    """
    Combine FATCAT output with CSV data for statistical modeling.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        prefix: Prefix for output files.
        csv_file: Comma-separated variable (CSV) file for input.
        matrix_files: Set of matrix (*.grid or *.netcc) files by searchable\
            path.
        list_match: Text file containing two columns: path to subject matrix\
            file and CSV IDs.
        unionize_rois: Make the ROI list as the union of elements across the\
            group.
        na_warn_off: Turn off the automatic warnings as the data table is\
            created.
        extern_labels_no: Turn off the writing/usage of user-defined labels in\
            the *.grid/*.netcc files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FatMvmPrepOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FAT_MVM_PREP_METADATA)
    cargs = []
    cargs.append("fat_mvm_prep.py")
    cargs.extend([
        "-p",
        prefix
    ])
    cargs.extend([
        "-c",
        execution.input_file(csv_file)
    ])
    if matrix_files is not None:
        cargs.extend([
            "-m",
            matrix_files
        ])
    if list_match is not None:
        cargs.extend([
            "-l",
            execution.input_file(list_match)
        ])
    if unionize_rois:
        cargs.append("-u")
    if na_warn_off:
        cargs.append("-N")
    if extern_labels_no:
        cargs.append("-E")
    ret = FatMvmPrepOutputs(
        root=execution.output_file("."),
        mvmtbl=execution.output_file(prefix + "_MVMtbl.txt"),
        mvmprep_log=execution.output_file(prefix + "_MVMprep.log"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FAT_MVM_PREP_METADATA",
    "FatMvmPrepOutputs",
    "fat_mvm_prep",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ICO_SUPERSAMPLE_METADATA = Metadata(
    id="14aa48c095a4ad3f4f834bf5d2084bdf9092540a.boutiques",
    name="ico_supersample",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class IcoSupersampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ico_supersample(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """The output file containing the refined icosahedron mesh."""


def ico_supersample(
    refine: bool = False,
    radius: float | None = None,
    projection_point: list[float] | None = None,
    runner: Runner | None = None,
) -> IcoSupersampleOutputs:
    """
    A tool for refining icosahedron meshes with user-specified parameters.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        refine: Refine the icosahedron mesh.
        radius: Radius of the sphere onto which the icosahedron is projected.
        projection_point: Projection point for the icosahedron refinement.\
            Enter three floating point values separated by spaces.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `IcoSupersampleOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ICO_SUPERSAMPLE_METADATA)
    cargs = []
    cargs.append("ico_supersample")
    if refine:
        cargs.append("-y")
    if radius is not None:
        cargs.append(str(radius))
    if projection_point is not None:
        cargs.extend(map(str, projection_point))
    ret = IcoSupersampleOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file("refined_icosahedron.obj"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ICO_SUPERSAMPLE_METADATA",
    "IcoSupersampleOutputs",
    "ico_supersample",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SEGMENT_HA_T1_LONG_SH_METADATA = Metadata(
    id="e6413cb2b0b0c7d6ddbeca885e2ef04a50aea294.boutiques",
    name="segmentHA_T1_long.sh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class SegmentHaT1LongShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `segment_ha_t1_long_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dir: OutputPathType
    """Output directory containing segmentation results"""


def segment_ha_t1_long_sh(
    subject_dir: str,
    subject_id: str,
    runner: Runner | None = None,
) -> SegmentHaT1LongShOutputs:
    """
    A script for longitudinal segmentation of the hippocampal/amygdala regions.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject_dir: Directory containing subject data.
        subject_id: Identifier for the subject within the subject directory.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SegmentHaT1LongShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SEGMENT_HA_T1_LONG_SH_METADATA)
    cargs = []
    cargs.append("segmentHA_T1_long.sh")
    cargs.append(subject_dir)
    cargs.append(subject_id)
    ret = SegmentHaT1LongShOutputs(
        root=execution.output_file("."),
        output_dir=execution.output_file(subject_dir + "/" + subject_id + "_long_segment/output"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SEGMENT_HA_T1_LONG_SH_METADATA",
    "SegmentHaT1LongShOutputs",
    "segment_ha_t1_long_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SCENE_FILE_RELOCATE_METADATA = Metadata(
    id="4d5785c213ba8c43cc0d1cb433a896970dc23e80.boutiques",
    name="scene-file-relocate",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class SceneFileRelocateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `scene_file_relocate(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def scene_file_relocate(
    input_scene: str,
    output_scene: str,
    runner: Runner | None = None,
) -> SceneFileRelocateOutputs:
    """
    Recreate scene file in new location.
    
    Scene files contain internal relative paths, such that moving or copying a
    scene file will cause it to lose track of the files it refers to. This
    command makes a modified copy of the scene file, changing the relative paths
    to refer to the new relative locations of the files.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        input_scene: the scene file to use.
        output_scene: output - the new scene file to create.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileRelocateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCENE_FILE_RELOCATE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-scene-file-relocate")
    cargs.append(input_scene)
    cargs.append(output_scene)
    ret = SceneFileRelocateOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SCENE_FILE_RELOCATE_METADATA",
    "SceneFileRelocateOutputs",
    "scene_file_relocate",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BBLABEL_METADATA = Metadata(
    id="8029e18ce532d6970cd367d113a7d2298a390304.boutiques",
    name="bblabel",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class BblabelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `bblabel(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output file with the label points within the specified bounding box."""


def bblabel(
    labelfile: InputPathType,
    outlabelfile: str,
    xmin: float | None = None,
    xmax: float | None = None,
    ymin: float | None = None,
    ymax: float | None = None,
    zmin: float | None = None,
    zmax: float | None = None,
    debug: bool = False,
    umask: str | None = None,
    runner: Runner | None = None,
) -> BblabelOutputs:
    """
    Applies a bounding box to a label, copying only the label points within the
    specified box to the output.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        labelfile: Input label file.
        outlabelfile: Output label file.
        xmin: Minimum x-coordinate for bounding box.
        xmax: Maximum x-coordinate for bounding box.
        ymin: Minimum y-coordinate for bounding box.
        ymax: Maximum y-coordinate for bounding box.
        zmin: Minimum z-coordinate for bounding box.
        zmax: Maximum z-coordinate for bounding box.
        debug: Enable debug mode.
        umask: Set Unix file permission mask.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BblabelOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BBLABEL_METADATA)
    cargs = []
    cargs.append("bblabel")
    cargs.extend([
        "--l",
        execution.input_file(labelfile)
    ])
    if xmin is not None:
        cargs.extend([
            "--xmin",
            str(xmin)
        ])
    if xmax is not None:
        cargs.extend([
            "--xmax",
            str(xmax)
        ])
    if ymin is not None:
        cargs.extend([
            "--ymin",
            str(ymin)
        ])
    if ymax is not None:
        cargs.extend([
            "--ymax",
            str(ymax)
        ])
    if zmin is not None:
        cargs.extend([
            "--zmin",
            str(zmin)
        ])
    if zmax is not None:
        cargs.extend([
            "--zmax",
            str(zmax)
        ])
    cargs.extend([
        "--o",
        outlabelfile
    ])
    if debug:
        cargs.append("--debug")
    if umask is not None:
        cargs.extend([
            "--umask",
            umask
        ])
    ret = BblabelOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(outlabelfile),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BBLABEL_METADATA",
    "BblabelOutputs",
    "bblabel",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_BRICK_STAT_METADATA = Metadata(
    id="2171879e0c09931b6800be575110e0354a0a8e86.boutiques",
    name="3dBrickStat",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dBrickStatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_brick_stat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    console_output: OutputPathType
    """Console output of computed statistics"""


def v_3d_brick_stat(
    dataset: str,
    quick: bool = False,
    slow: bool = False,
    min_: bool = False,
    max_: bool = False,
    mean: bool = False,
    sum_: bool = False,
    var: bool = False,
    stdev: bool = False,
    count: bool = False,
    volume: bool = False,
    positive: bool = False,
    negative: bool = False,
    zero: bool = False,
    non_positive: bool = False,
    non_negative: bool = False,
    non_zero: bool = False,
    absolute: bool = False,
    nan: bool = False,
    nonan: bool = False,
    mask: str | None = None,
    mrange: list[float] | None = None,
    mvalue: float | None = None,
    automask: bool = False,
    percentile: list[float] | None = None,
    perclist: list[float] | None = None,
    median: bool = False,
    perc_quiet: bool = False,
    ver: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> V3dBrickStatOutputs:
    """
    Compute voxel statistics of an input dataset.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dataset: Input dataset.
        quick: Get the information from the header only (default).
        slow: Read the whole dataset to find the min and max values.
        min_: Print the minimum value in dataset.
        max_: Print the maximum value in dataset (default).
        mean: Print the mean value in dataset.
        sum_: Print the sum of values in the dataset.
        var: Print the variance in the dataset.
        stdev: Print the standard deviation in the dataset.
        count: Print the number of voxels included.
        volume: Print the volume of voxels included in microliters.
        positive: Include only positive voxel values.
        negative: Include only negative voxel values.
        zero: Include only zero voxel values.
        non_positive: Include only voxel values 0 or negative.
        non_negative: Include only voxel values 0 or greater.
        non_zero: Include only voxel values not equal to 0.
        absolute: Use absolute value of voxel values for all calculations.
        nan: Include only voxel values that are NaN or inf. Forces -slow mode.
        nonan: Exclude voxel values that are NaN or inf.
        mask: Use the specified dataset as mask to include/exclude voxels.
        mrange: Only accept values between MIN and MAX (inclusive) from the\
            mask.
        mvalue: Only accept values equal to VAL from the mask.
        automask: Automatically compute mask for dataset. Cannot be combined\
            with -mask.
        percentile: Compute and print percentile values from p0% to p1% at a\
            step of ps%. Only one sub-brick is accepted as input with this option.
        perclist: Like -percentile, but output the given percentiles.
        median: Shortcut for -percentile 50 1 50 (or -perclist 1 50).
        perc_quiet: Only print percentile results, not input percentile cutoffs.
        ver: Print author and version info.
        help_: Print help screen.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dBrickStatOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_BRICK_STAT_METADATA)
    cargs = []
    cargs.append("3dBrickStat")
    cargs.append(dataset)
    if quick:
        cargs.append("-quick")
    if slow:
        cargs.append("-slow")
    if min_:
        cargs.append("-min")
    if max_:
        cargs.append("-max")
    if mean:
        cargs.append("-mean")
    if sum_:
        cargs.append("-sum")
    if var:
        cargs.append("-var")
    if stdev:
        cargs.append("-stdev")
    if count:
        cargs.append("-count")
    if volume:
        cargs.append("-volume")
    if positive:
        cargs.append("-positive")
    if negative:
        cargs.append("-negative")
    if zero:
        cargs.append("-zero")
    if non_positive:
        cargs.append("-non-positive")
    if non_negative:
        cargs.append("-non-negative")
    if non_zero:
        cargs.append("-non-zero")
    if absolute:
        cargs.append("-absolute")
    if nan:
        cargs.append("-nan")
    if nonan:
        cargs.append("-nonan")
    if mask is not None:
        cargs.extend([
            "-mask",
            mask
        ])
    if mrange is not None:
        cargs.extend([
            "-mrange",
            *map(str, mrange)
        ])
    if mvalue is not None:
        cargs.extend([
            "-mvalue",
            str(mvalue)
        ])
    if automask:
        cargs.append("-automask")
    if percentile is not None:
        cargs.extend([
            "-percentile",
            *map(str, percentile)
        ])
    if perclist is not None:
        cargs.extend([
            "-perclist",
            *map(str, perclist)
        ])
    if median:
        cargs.append("-median")
    if perc_quiet:
        cargs.append("-perc_quiet")
    if ver:
        cargs.append("-ver")
    if help_:
        cargs.append("-help")
    ret = V3dBrickStatOutputs(
        root=execution.output_file("."),
        console_output=execution.output_file("output.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dBrickStatOutputs",
    "V_3D_BRICK_STAT_METADATA",
    "v_3d_brick_stat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SEGMENT_HA_T2_SH_METADATA = Metadata(
    id="e35fef1406f0bc21d5d98039621ba41ed9020379.boutiques",
    name="segmentHA_T2.sh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class SegmentHaT2ShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `segment_ha_t2_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    left_hemisphere_labels: OutputPathType
    """Left hemisphere hippocampal and amygdala labels"""
    right_hemisphere_labels: OutputPathType
    """Right hemisphere hippocampal and amygdala labels"""


def segment_ha_t2_sh(
    input_image: InputPathType,
    output_directory: str,
    runner: Runner | None = None,
) -> SegmentHaT2ShOutputs:
    """
    Segments hippocampal and amygdala structures from T2-weighted MRI images using
    the FreeSurfer suite.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_image: Input T2-weighted MRI image file.
        output_directory: Output directory for segmented structures.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SegmentHaT2ShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SEGMENT_HA_T2_SH_METADATA)
    cargs = []
    cargs.append("segmentHA_T2.sh")
    cargs.append(execution.input_file(input_image))
    cargs.append(output_directory)
    ret = SegmentHaT2ShOutputs(
        root=execution.output_file("."),
        left_hemisphere_labels=execution.output_file(output_directory + "/lh.hippoAmygLabels-T2.v21.mgz"),
        right_hemisphere_labels=execution.output_file(output_directory + "/rh.hippoAmygLabels-T2.v21.mgz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SEGMENT_HA_T2_SH_METADATA",
    "SegmentHaT2ShOutputs",
    "segment_ha_t2_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURF_SMOOTH_METADATA = Metadata(
    id="9567232249acc1b0e69c280f3ce45f44bb256c86.boutiques",
    name="SurfSmooth",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class SurfSmoothOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surf_smooth(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_file: OutputPathType | None
    """Name of the output file."""


def surf_smooth(
    surface: str,
    method: str,
    input_data: InputPathType | None = None,
    target_fwhm: float | None = None,
    fwhm: float | None = None,
    number_iterations: float | None = None,
    output_file: InputPathType | None = None,
    band_pass_frequency: float | None = None,
    lambda_mu: str | None = None,
    interp_weights: str | None = None,
    node_mask: InputPathType | None = None,
    surface_output: InputPathType | None = None,
    dbg_node: float | None = None,
    use_neighbors_outside_mask: bool = False,
    talk_suma: bool = False,
    refresh_rate: float | None = None,
    runner: Runner | None = None,
) -> SurfSmoothOutputs:
    """
    Tool for smoothing data on surfaces using various methods.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        surface: Option for specifying the surface to smooth or the domain over\
            which DSET is defined.
        method: Name of smoothing method to use. Choose from: HEAT_07, HEAT_05,\
            LM, NN_geom.
        input_data: File containing data (in 1D or NIML format). Required for\
            HEAT_05 and HEAT_07 methods.
        target_fwhm: Blur so that the final FWHM of the data is TF mm. Only for\
            HEAT_07 method.
        fwhm: Effective Full Width at Half Maximum for smoothing. Required for\
            HEAT_05 and optional for HEAT_07 methods.
        number_iterations: Number of smoothing iterations (default is 100 for\
            LM and NN_geom, -1 for HEAT methods).
        output_file: Name of output file. Default based on method being used.
        band_pass_frequency: Bandpass frequency for LM method (0 < k < 10).
        lambda_mu: Lambda and Mu parameters for LM method. Sample values are:\
            0.6307 and -0.6732.
        interp_weights: Set interpolation weights for LM method. Options:\
            Equal, Fujiwara, Desbrun.
        node_mask: Apply operations only to nodes listed in the given mask.
        surface_output: Writes the surface with smoothed coordinates to disk.\
            For LM and NN_geom methods.
        dbg_node: Output debug information for node 'node'.
        use_neighbors_outside_mask: Allow value from a node neighboring node n\
            to contribute to the value at n even if the neighbor is not in the\
            mask.
        talk_suma: Send progress with each iteration to SUMA for real-time\
            visualization.
        refresh_rate: Maximum number of updates to SUMA per second.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfSmoothOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURF_SMOOTH_METADATA)
    cargs = []
    cargs.append("SurfSmooth")
    cargs.extend([
        "-SURF_1",
        surface
    ])
    cargs.extend([
        "-met",
        method
    ])
    if input_data is not None:
        cargs.extend([
            "-input",
            execution.input_file(input_data)
        ])
    if target_fwhm is not None:
        cargs.extend([
            "-target_fwhm",
            str(target_fwhm)
        ])
    if fwhm is not None:
        cargs.extend([
            "-fwhm",
            str(fwhm)
        ])
    if number_iterations is not None:
        cargs.extend([
            "-Niter",
            str(number_iterations)
        ])
    if output_file is not None:
        cargs.extend([
            "-output",
            execution.input_file(output_file)
        ])
    if band_pass_frequency is not None:
        cargs.extend([
            "-kpb",
            str(band_pass_frequency)
        ])
    if lambda_mu is not None:
        cargs.extend([
            "-lm",
            lambda_mu
        ])
    if interp_weights is not None:
        cargs.extend([
            "-iw",
            interp_weights
        ])
    if node_mask is not None:
        cargs.extend([
            "-MASK",
            execution.input_file(node_mask)
        ])
    if surface_output is not None:
        cargs.extend([
            "-surf_out",
            execution.input_file(surface_output)
        ])
    if dbg_node is not None:
        cargs.extend([
            "-dbg_n",
            str(dbg_node)
        ])
    if use_neighbors_outside_mask:
        cargs.append("-use_neighbors_outside_mask")
    if talk_suma:
        cargs.append("-talk_suma")
    if refresh_rate is not None:
        cargs.extend([
            "-refresh_rate",
            str(refresh_rate)
        ])
    ret = SurfSmoothOutputs(
        root=execution.output_file("."),
        out_file=execution.output_file(pathlib.Path(output_file).name) if (output_file is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURF_SMOOTH_METADATA",
    "SurfSmoothOutputs",
    "surf_smooth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

GCAINIT_METADATA = Metadata(
    id="00dfdd96c6b86f2b1c5b47fc4b41253a5bcd0e9f.boutiques",
    name="gcainit",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class GcainitOutputs(typing.NamedTuple):
    """
    Output object returned when calling `gcainit(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def gcainit(
    gcadir: str,
    runner: Runner | None = None,
) -> GcainitOutputs:
    """
    Initializes the GCA for brain processing tasks.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        gcadir: Output directory of gcaprep.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GcainitOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(GCAINIT_METADATA)
    cargs = []
    cargs.append("gcainit")
    cargs.extend([
        "--g",
        gcadir
    ])
    ret = GcainitOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "GCAINIT_METADATA",
    "GcainitOutputs",
    "gcainit",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANNOT2STD_METADATA = Metadata(
    id="fcdd66f979c54df988b9364c89c1d5056e322ab6.boutiques",
    name="annot2std",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class Annot2stdOutputs(typing.NamedTuple):
    """
    Output object returned when calling `annot2std(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_annot_file: OutputPathType
    """Main output annotation file"""
    out_prob_map_file: OutputPathType
    """Output annotation probability map file"""
    output_surface_segmentation: OutputPathType | None
    """Output surface segmentation file"""


def annot2std(
    output_annot_path: str,
    subjects: list[str],
    fsgd_file: InputPathType | None = None,
    subject_list_file: InputPathType | None = None,
    target: str | None = None,
    right_hemisphere: bool = False,
    xhemi: bool = False,
    surfreg: str | None = None,
    srcsurfreg: str | None = None,
    trgsurfreg: str | None = None,
    a2009s: bool = False,
    segmentation: str | None = None,
    stack: str | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Annot2stdOutputs:
    """
    Creates an average annotation in a standard space based on transforming the
    annotations of the individual subjects to the standard space through the surface
    registration.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        output_annot_path: Full output annotation path (also creates\
            outannotpath.p.mgh).
        subjects: Input subject(s), specify each subject with --s subj.
        fsgd_file: FSGD file for group descriptor.
        subject_list_file: Subject list file.
        target: Target subject (e.g., fsaverage).
        right_hemisphere: Use right hemisphere.
        xhemi: For interhemispheric analysis.
        surfreg: Surface registration type (default is sphere.reg).
        srcsurfreg: Source surface registration type (default is sphere.reg).
        trgsurfreg: Target surface registration type (default is sphere.reg).
        a2009s: Annotation name set to aparc.a2009s.
        segmentation: Save output as a surface segmentation (2 frames, second =\
            p).
        stack: Stack of individual annotations as segmentation.
        help_: Display help.
        version: Display version.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Annot2stdOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANNOT2STD_METADATA)
    cargs = []
    cargs.append("annot2std")
    cargs.extend([
        "--o",
        output_annot_path
    ])
    cargs.extend([
        "--s",
        *subjects
    ])
    if fsgd_file is not None:
        cargs.extend([
            "--fsgd",
            execution.input_file(fsgd_file)
        ])
    if subject_list_file is not None:
        cargs.extend([
            "--f",
            execution.input_file(subject_list_file)
        ])
    if target is not None:
        cargs.extend([
            "--t",
            target
        ])
    if right_hemisphere:
        cargs.append("--rh")
    if xhemi:
        cargs.append("--xhemi")
    if surfreg is not None:
        cargs.extend([
            "--surfreg",
            surfreg
        ])
    if srcsurfreg is not None:
        cargs.extend([
            "--srcsurfreg",
            srcsurfreg
        ])
    if trgsurfreg is not None:
        cargs.extend([
            "--trgsurfreg",
            trgsurfreg
        ])
    if a2009s:
        cargs.append("--a2009s")
    if segmentation is not None:
        cargs.extend([
            "--seg",
            segmentation
        ])
    if stack is not None:
        cargs.extend([
            "--stack",
            stack
        ])
    if help_:
        cargs.append("--help")
    if version:
        cargs.append("--version")
    ret = Annot2stdOutputs(
        root=execution.output_file("."),
        out_annot_file=execution.output_file(output_annot_path),
        out_prob_map_file=execution.output_file(output_annot_path + ".p.mgh"),
        output_surface_segmentation=execution.output_file(segmentation) if (segmentation is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANNOT2STD_METADATA",
    "Annot2stdOutputs",
    "annot2std",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_ANATOMICAL_STATS_METADATA = Metadata(
    id="26651665f8aa89f4302f7661390895c29f6b22f4.boutiques",
    name="mris_anatomical_stats",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisAnatomicalStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_anatomical_stats(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_log_file: OutputPathType | None
    """Output log file of the statistics"""
    output_table_file: OutputPathType | None
    """Tabular output stored in a table file"""
    output_ctab_file: OutputPathType | None
    """Output annotation color table file"""


def mris_anatomical_stats(
    subjectname: str,
    hemisphere: str,
    surfacename: str | None = None,
    thickness_range: list[float] | None = None,
    label_file: InputPathType | None = None,
    thickness_file: InputPathType | None = None,
    annotation_file: InputPathType | None = None,
    tabular_output: bool = False,
    tablefile: str | None = None,
    logfile: str | None = None,
    nsmooth: float | None = None,
    color_table: str | None = None,
    noglobal: bool = False,
    th3_computation: bool = False,
    runner: Runner | None = None,
) -> MrisAnatomicalStatsOutputs:
    """
    This program computes a number of anatomical properties.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjectname: Subject name.
        hemisphere: Hemisphere.
        surfacename: Surface name.
        thickness_range: Only consider thicknesses in the specified range.
        label_file: Limit calculations to specified label.
        thickness_file: Use specified file for computing thickness statistics.
        annotation_file: Compute properties for each label in the annotation\
            file separately.
        tabular_output: Tabular output.
        tablefile: Table output to tablefile. Must use -a or -l options to\
            specify input.
        logfile: Write stats to file named log.
        nsmooth: Smooth thickness map # of iterations before using it.
        color_table: Output annotation file's color table to text file.
        noglobal: Do not compute global brain stats.
        th3_computation: Compute vertex-wise volume by dividing each obliquely\
            truncated trilateral pyramid into three tetrahedra.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisAnatomicalStatsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_ANATOMICAL_STATS_METADATA)
    cargs = []
    cargs.append("mris_anatomical_stats")
    cargs.append(subjectname)
    cargs.append(hemisphere)
    if surfacename is not None:
        cargs.append(surfacename)
    if thickness_range is not None:
        cargs.extend([
            "-i",
            *map(str, thickness_range)
        ])
    if label_file is not None:
        cargs.extend([
            "-l",
            execution.input_file(label_file)
        ])
    if thickness_file is not None:
        cargs.extend([
            "-t",
            execution.input_file(thickness_file)
        ])
    if annotation_file is not None:
        cargs.extend([
            "-a",
            execution.input_file(annotation_file)
        ])
    if tabular_output:
        cargs.append("-b")
    if tablefile is not None:
        cargs.extend([
            "-f",
            tablefile
        ])
    if logfile is not None:
        cargs.extend([
            "-log",
            logfile
        ])
    if nsmooth is not None:
        cargs.extend([
            "-nsmooth",
            str(nsmooth)
        ])
    if color_table is not None:
        cargs.extend([
            "-c",
            color_table
        ])
    if noglobal:
        cargs.append("-noglobal")
    if th3_computation:
        cargs.append("-th3")
    ret = MrisAnatomicalStatsOutputs(
        root=execution.output_file("."),
        output_log_file=execution.output_file(logfile + ".txt") if (logfile is not None) else None,
        output_table_file=execution.output_file(tablefile + ".txt") if (tablefile is not None) else None,
        output_ctab_file=execution.output_file(color_table + ".txt") if (color_table is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_ANATOMICAL_STATS_METADATA",
    "MrisAnatomicalStatsOutputs",
    "mris_anatomical_stats",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BROWSE_MINC_HEADER_TCL_METADATA = Metadata(
    id="d9e64b68673469631420a82f8e3342cb78163bcc.boutiques",
    name="browse-minc-header.tcl",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class BrowseMincHeaderTclOutputs(typing.NamedTuple):
    """
    Output object returned when calling `browse_minc_header_tcl(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def browse_minc_header_tcl(
    infile: InputPathType,
    runner: Runner | None = None,
) -> BrowseMincHeaderTclOutputs:
    """
    A tool for browsing MINC file headers, likely part of the FreeSurfer package.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        infile: MINC file for which the header is to be browsed.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BrowseMincHeaderTclOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BROWSE_MINC_HEADER_TCL_METADATA)
    cargs = []
    cargs.append("browse-minc-header.tcl")
    cargs.append(execution.input_file(infile))
    ret = BrowseMincHeaderTclOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BROWSE_MINC_HEADER_TCL_METADATA",
    "BrowseMincHeaderTclOutputs",
    "browse_minc_header_tcl",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

APPLY_MORPH_METADATA = Metadata(
    id="a7cb7e420a1c51fbb13f7cd6c2a1a9581804bf02.boutiques",
    name="applyMorph",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class ApplyMorphOutputs(typing.NamedTuple):
    """
    Output object returned when calling `apply_morph(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def apply_morph(
    inputs: list[InputPathType],
    template: InputPathType,
    transform: InputPathType,
    zlib_buffer: float | None = None,
    dbg_coords: list[float] | None = None,
    runner: Runner | None = None,
) -> ApplyMorphOutputs:
    """
    A tool for applying a morph to a volume using a template and a transform.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        inputs: Input files.
        template: Template volume.
        transform: Transform file.
        zlib_buffer: Zlib buffer pre-allocation multiplier.
        dbg_coords: Debugging coordinates.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ApplyMorphOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(APPLY_MORPH_METADATA)
    cargs = []
    cargs.append("applyMorph")
    cargs.extend([execution.input_file(f) for f in inputs])
    cargs.extend([
        "--template",
        execution.input_file(template)
    ])
    cargs.extend([
        "--transform",
        execution.input_file(transform)
    ])
    if zlib_buffer is not None:
        cargs.extend([
            "--zlib_buffer",
            str(zlib_buffer)
        ])
    if dbg_coords is not None:
        cargs.extend([
            "--dbg_coords",
            *map(str, dbg_coords)
        ])
    ret = ApplyMorphOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "APPLY_MORPH_METADATA",
    "ApplyMorphOutputs",
    "apply_morph",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SIMULATE_DISPLACEMENT_FIELD_METADATA = Metadata(
    id="b4b4d74766ee9ec6eddb1c050e1dc66d6fc3cd4d.boutiques",
    name="SimulateDisplacementField",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


@dataclasses.dataclass
class SimulateDisplacementFieldBsplineOptions:
    number_of_fitting_levels: int | None = 4
    """Number of fitting levels for BSpline."""
    number_of_control_points: int | None = 4
    """Number of control points for BSpline."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.number_of_fitting_levels is not None:
            cargs.append(str(self.number_of_fitting_levels))
        if self.number_of_control_points is not None:
            cargs.append(str(self.number_of_control_points))
        return cargs


@dataclasses.dataclass
class SimulateDisplacementFieldExponentialOptions:
    smoothing_standard_deviation: float | None = 4
    """Smoothing standard deviation for Exponential."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.smoothing_standard_deviation is not None:
            cargs.append(str(self.smoothing_standard_deviation))
        return cargs


class SimulateDisplacementFieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `simulate_displacement_field(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_displacement_field: OutputPathType
    """The simulated displacement field."""


def simulate_displacement_field(
    image_dimension: int,
    displacement_field_type: typing.Literal["BSpline", "Exponential"],
    domain_image: InputPathType,
    output_field: InputPathType,
    number_of_random_points: int | None = 1000,
    standard_deviation_displacement_field: float | None = 10,
    enforce_stationary_boundary: int | None = 1,
    displacement_specific_options: typing.Union[SimulateDisplacementFieldBsplineOptions, SimulateDisplacementFieldExponentialOptions] | None = None,
    runner: Runner | None = None,
) -> SimulateDisplacementFieldOutputs:
    """
    Simulate displacement fields using various methods such as BSpline or
    Exponential.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Dimensionality of the image.
        displacement_field_type: Type of displacement field to simulate.
        domain_image: Image defining the domain for the displacement field.
        output_field: Path to save the output displacement field.
        number_of_random_points: Number of random points to use in the\
            simulation.
        standard_deviation_displacement_field: Standard deviation for the\
            displacement field.
        enforce_stationary_boundary: Boolean flag indicating whether to enforce\
            stationary boundary.
        displacement_specific_options: Options specific to the type of\
            displacement field simulation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SimulateDisplacementFieldOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SIMULATE_DISPLACEMENT_FIELD_METADATA)
    cargs = []
    cargs.append("SimulateDisplacementField")
    cargs.append(str(image_dimension))
    cargs.append(displacement_field_type)
    cargs.append(execution.input_file(domain_image))
    cargs.append(execution.input_file(output_field))
    if number_of_random_points is not None:
        cargs.append(str(number_of_random_points))
    if standard_deviation_displacement_field is not None:
        cargs.append(str(standard_deviation_displacement_field))
    if enforce_stationary_boundary is not None:
        cargs.append(str(enforce_stationary_boundary))
    if displacement_specific_options is not None:
        cargs.extend(displacement_specific_options.run(execution))
    ret = SimulateDisplacementFieldOutputs(
        root=execution.output_file("."),
        output_displacement_field=execution.output_file(pathlib.Path(output_field).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SIMULATE_DISPLACEMENT_FIELD_METADATA",
    "SimulateDisplacementFieldBsplineOptions",
    "SimulateDisplacementFieldExponentialOptions",
    "SimulateDisplacementFieldOutputs",
    "simulate_displacement_field",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_LABEL2ANNOT_METADATA = Metadata(
    id="67f30d66c4a858ea344c43f9704f2d647b6aa35d.boutiques",
    name="mris_label2annot",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisLabel2annotOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_label2annot(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    annot_file: OutputPathType
    """Generated annotation file"""


def mris_label2annot(
    subject: str,
    hemi: str,
    ctabfile: InputPathType,
    annotname: str,
    index_offset: float | None = None,
    label_files: list[InputPathType] | None = None,
    annot_path: str | None = None,
    labeldir: str | None = None,
    ldir_default: bool = False,
    no_unknown: bool = False,
    thresh: float | None = None,
    maxstatwinner: bool = False,
    surf: str | None = None,
    subjects_dir: str | None = None,
    runner: Runner | None = None,
) -> MrisLabel2annotOutputs:
    """
    Converts a set of surface labels to an annotation file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: FreeSurfer subject.
        hemi: Hemisphere (lh or rh).
        ctabfile: Colortable file (like FreeSurferColorLUT.txt).
        annotname: Output annotation name.
        index_offset: Add to label number to get CTAB index.
        label_files: Label file(s).
        annot_path: Full name/path of annotation file.
        labeldir: Directory with label files when not using --l.
        ldir_default: Use subject/labels as label directory.
        no_unknown: Do not map unhit labels to index 0.
        thresh: Threshold label by stats field.
        maxstatwinner: Keep label with highest 'stat' value.
        surf: Surface name, default is orig.
        subjects_dir: Subjects Directory.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisLabel2annotOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_LABEL2ANNOT_METADATA)
    cargs = []
    cargs.append("mris_label2annot")
    cargs.extend([
        "-s",
        subject
    ])
    cargs.extend([
        "-h",
        hemi
    ])
    cargs.extend([
        "-ctab",
        execution.input_file(ctabfile)
    ])
    cargs.extend([
        "-a",
        annotname
    ])
    if index_offset is not None:
        cargs.extend([
            "--offset",
            str(index_offset)
        ])
    if label_files is not None:
        cargs.extend([
            "--l",
            *[execution.input_file(f) for f in label_files]
        ])
    if annot_path is not None:
        cargs.extend([
            "--annot-path",
            annot_path
        ])
    if labeldir is not None:
        cargs.extend([
            "--ldir",
            labeldir
        ])
    if ldir_default:
        cargs.append("--ldir-default")
    if no_unknown:
        cargs.append("--no-unknown")
    if thresh is not None:
        cargs.extend([
            "--thresh",
            str(thresh)
        ])
    if maxstatwinner:
        cargs.append("--maxstatwinner")
    if surf is not None:
        cargs.extend([
            "--surf",
            surf
        ])
    if subjects_dir is not None:
        cargs.extend([
            "--sd",
            subjects_dir
        ])
    ret = MrisLabel2annotOutputs(
        root=execution.output_file("."),
        annot_file=execution.output_file(hemi + "." + annotname + ".annot"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_LABEL2ANNOT_METADATA",
    "MrisLabel2annotOutputs",
    "mris_label2annot",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_GLMFIT_SIM_METADATA = Metadata(
    id="b485cfa7c95293942f2efa8d5ccde5c8c33116e9.boutiques",
    name="mri_glmfit-sim",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriGlmfitSimOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_glmfit_sim(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    sig_voxel_mgh: OutputPathType | None
    """Sig volume corrected for voxel-wise multiple comparisons."""
    sig_cluster_mgh: OutputPathType | None
    """Sig volume corrected for cluster-wise multiple comparisons."""
    sig_cluster_summary: OutputPathType | None
    """Cluster summary table."""
    y_ocn_dat: OutputPathType | None
    """Summary of the input (y) over each cluster."""
    sig_ocn_mgh: OutputPathType | None
    """Output cluster number volume."""
    sig_ocn_annot: OutputPathType | None
    """Cluster annotation for surfaces."""
    sig_masked_mgh: OutputPathType | None
    """Original sig volume masked to show only clusters."""


def mri_glmfit_sim(
    glmdir: str,
    cwp: float | None = None,
    mczsim: str | None = None,
    mczsim_dir: str | None = None,
    mczsim_label: str | None = None,
    perm: str | None = None,
    perm_resid: bool = False,
    perm_signflip: bool = False,
    grf: str | None = None,
    spaces_2: bool = False,
    spaces_3: bool = False,
    overwrite: bool = False,
    bg: float | None = None,
    sleep: float | None = None,
    a2009s: bool = False,
    annot: str | None = None,
    log: str | None = None,
    base: str | None = None,
    no_sim: str | None = None,
    seed: float | None = None,
    fwhm_override: float | None = None,
    fwhm_add: float | None = None,
    uniform: list[float] | None = None,
    no_out_annot: bool = False,
    no_cluster_mean: bool = False,
    y_file: InputPathType | None = None,
    centroid: bool = False,
    spatial_sum: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> MriGlmfitSimOutputs:
    """
    A tool to run corrections for multiple comparisons on volumes or surfaces, using
    various methods including Monte Carlo simulation, permutation, and Gaussian
    Random Fields.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        glmdir: Path to GLM directory.
        cwp: Cluster-wise p-value threshold, default is 0.05.
        mczsim: Use pre-computed z-based Monte Carlo simulations. Requires\
            vwthreshold and sign.
        mczsim_dir: Directory for custom Monte Carlo simulations, default is\
            FREESURFER_HOME/average/mult-comp-cor.
        mczsim_label: Label for custom Monte Carlo simulations, default is\
            cortex.
        perm: Use permutation simulation with nsim iterations, cluster forming\
            threshold (CFT), and sign.
        perm_resid: Use permutation on the residual instead of raw data for\
            non-orthogonal designs.
        perm_signflip: Use sign flipping instead of shuffling for permutation.
        grf: Use Gaussian Random Fields (GRF) method, with vwthreshold and\
            sign.
        spaces_2: Apply additional Bonferroni correction across 2 spaces.
        spaces_3: Apply additional Bonferroni correction across 3 spaces.
        overwrite: Delete previous CSDs.
        bg: Divide simulation into njobs and put in background.
        sleep: Number of seconds to sleep between background polls.
        a2009s: Use aparc.a2009s instead of aparc for region of vertex max.
        annot: Use specific annotation for region of vertex max.
        log: Specify logfile, default is csdbase.mri_glmfit-sim.log.
        base: Override csdbase name.
        no_sim: Do not simulate, only run cluster.
        seed: Set simulation random number generator seed.
        fwhm_override: Override fwhm in glmdir.
        fwhm_add: Add fwhmAdd to the estimated fwhm.
        uniform: Use uniform PDF instead of gaussian, specify min and max.
        no_out_annot: Do not output a cluster annotation.
        no_cluster_mean: Do not compute means of each subject in each cluster.
        y_file: Specify the GLM input y file.
        centroid: Report the coordinates/annotation of the centroid instead of\
            max.
        spatial_sum: Compute the sum over voxels in the cluster rather than the\
            average.
        help_: Show the help message and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriGlmfitSimOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_GLMFIT_SIM_METADATA)
    cargs = []
    cargs.append("mri_glmfit-sim")
    cargs.extend([
        "--glmdir",
        glmdir
    ])
    if cwp is not None:
        cargs.extend([
            "--cwp",
            str(cwp)
        ])
    if mczsim is not None:
        cargs.extend([
            "--mczsim",
            mczsim
        ])
    if mczsim_dir is not None:
        cargs.extend([
            "--mczsim-dir",
            mczsim_dir
        ])
    if mczsim_label is not None:
        cargs.extend([
            "--mczsim-label",
            mczsim_label
        ])
    if perm is not None:
        cargs.extend([
            "--perm",
            perm
        ])
    if perm_resid:
        cargs.append("--perm-resid")
    if perm_signflip:
        cargs.append("--perm-signflip")
    if grf is not None:
        cargs.extend([
            "--grf",
            grf
        ])
    if spaces_2:
        cargs.append("--2spaces")
    if spaces_3:
        cargs.append("--3spaces")
    if overwrite:
        cargs.append("--overwrite")
    if bg is not None:
        cargs.extend([
            "--bg",
            str(bg)
        ])
    if sleep is not None:
        cargs.extend([
            "--sleep",
            str(sleep)
        ])
    if a2009s:
        cargs.append("--a2009s")
    if annot is not None:
        cargs.extend([
            "--annot",
            annot
        ])
    if log is not None:
        cargs.extend([
            "--log",
            log
        ])
    if base is not None:
        cargs.extend([
            "--base",
            base
        ])
    if no_sim is not None:
        cargs.extend([
            "--no-sim",
            no_sim
        ])
    if seed is not None:
        cargs.extend([
            "--seed",
            str(seed)
        ])
    if fwhm_override is not None:
        cargs.extend([
            "--fwhm-override",
            str(fwhm_override)
        ])
    if fwhm_add is not None:
        cargs.extend([
            "--fwhm-add",
            str(fwhm_add)
        ])
    if uniform is not None:
        cargs.extend([
            "--uniform",
            *map(str, uniform)
        ])
    if no_out_annot:
        cargs.append("--no-out-annot")
    if no_cluster_mean:
        cargs.append("--no-cluster-mean")
    if y_file is not None:
        cargs.extend([
            "--y",
            execution.input_file(y_file)
        ])
    if centroid:
        cargs.append("--centroid")
    if spatial_sum:
        cargs.append("--spatial-sum")
    if help_:
        cargs.append("--help")
    ret = MriGlmfitSimOutputs(
        root=execution.output_file("."),
        sig_voxel_mgh=execution.output_file(base + ".sig.voxel.mgh") if (base is not None) else None,
        sig_cluster_mgh=execution.output_file(base + ".sig.cluster.mgh") if (base is not None) else None,
        sig_cluster_summary=execution.output_file(base + ".sig.cluster.summary") if (base is not None) else None,
        y_ocn_dat=execution.output_file(base + ".y.ocn.dat") if (base is not None) else None,
        sig_ocn_mgh=execution.output_file(base + ".sig.ocn.mgh") if (base is not None) else None,
        sig_ocn_annot=execution.output_file(base + ".sig.ocn.annot") if (base is not None) else None,
        sig_masked_mgh=execution.output_file(base + ".sig.masked.mgh") if (base is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_GLMFIT_SIM_METADATA",
    "MriGlmfitSimOutputs",
    "mri_glmfit_sim",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CHECK_MCR_SH_METADATA = Metadata(
    id="791978a63c49f038efea95973ac807e32f4fd537.boutiques",
    name="checkMCR.sh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class CheckMcrShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `check_mcr_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def check_mcr_sh(
    help_: bool = False,
    runner: Runner | None = None,
) -> CheckMcrShOutputs:
    """
    Script to check for the presence of Matlab Compiler Runtime (MCR) for Matlab
    2019b, used for the hippocampal/amygdala and brainstem modules.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        help_: Display help information about checkMCR.sh.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CheckMcrShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CHECK_MCR_SH_METADATA)
    cargs = []
    cargs.append("checkMCR.sh")
    if help_:
        cargs.append("--help")
    ret = CheckMcrShOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CHECK_MCR_SH_METADATA",
    "CheckMcrShOutputs",
    "check_mcr_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

AFNI_PROC_PY_METADATA = Metadata(
    id="a4908cc703567652b0ff66dd10430d6df6144142.boutiques",
    name="afni_proc.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class AfniProcPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `afni_proc_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_files: OutputPathType | None
    """All output files stored in the specified output directory."""


def afni_proc_py(
    dsets: list[InputPathType],
    subj_id: str,
    anat: InputPathType,
    out_dir: str | None = None,
    blocks: list[str] | None = None,
    echo_times: list[float] | None = None,
    stim_times: list[InputPathType] | None = None,
    stim_files: list[InputPathType] | None = None,
    copy_files: list[InputPathType] | None = None,
    copy_anat: InputPathType | None = None,
    regress_params: list[str] | None = None,
    runner: Runner | None = None,
) -> AfniProcPyOutputs:
    """
    Generate a tcsh script for an AFNI single subject processing stream.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dsets: Specify the EPI dataset files. (e.g. epi_run1+orig,\
            epi_run2+orig).
        subj_id: Specify the subject ID for the script.
        anat: Specify the anatomical dataset.
        out_dir: Specify the output directory for the script.
        blocks: Specify the processing blocks to apply (e.g. tshift volreg blur\
            mask scale regress).
        echo_times: Specify echo times for multi-echo data processing.
        stim_times: Specify files used for stimulus timing in -stim_times.
        stim_files: Specify TR-locked stim files for 3dDeconvolve -stim_file\
            instead of -stim_times.
        copy_files: Specify additional files to be copied to the results\
            directory.
        copy_anat: Copy the anatomical dataset(s) to the results directory.
        regress_params: Specify extra options for 3dDeconvolve.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AfniProcPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(AFNI_PROC_PY_METADATA)
    cargs = []
    cargs.append("afni_proc.py")
    cargs.extend([execution.input_file(f) for f in dsets])
    cargs.append(subj_id)
    if out_dir is not None:
        cargs.append(out_dir)
    if blocks is not None:
        cargs.extend(blocks)
    cargs.append(execution.input_file(anat))
    if echo_times is not None:
        cargs.extend(map(str, echo_times))
    if stim_times is not None:
        cargs.extend([execution.input_file(f) for f in stim_times])
    if stim_files is not None:
        cargs.extend([execution.input_file(f) for f in stim_files])
    if copy_files is not None:
        cargs.extend([execution.input_file(f) for f in copy_files])
    if copy_anat is not None:
        cargs.append(execution.input_file(copy_anat))
    if regress_params is not None:
        cargs.extend([
            "-regress_opts_3dD",
            *regress_params
        ])
    ret = AfniProcPyOutputs(
        root=execution.output_file("."),
        output_files=execution.output_file(out_dir + "/*") if (out_dir is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "AFNI_PROC_PY_METADATA",
    "AfniProcPyOutputs",
    "afni_proc_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

METRIC_FALSE_CORRELATION_METADATA = Metadata(
    id="1caaae17d3f0c5a13c0d0ecb9b74ca6b5a6dc29b.boutiques",
    name="metric-false-correlation",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class MetricFalseCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `metric_false_correlation(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_false_correlation(
    surface: InputPathType,
    metric_in: InputPathType,
    v_3d_dist: float,
    geo_outer: float,
    geo_inner: float,
    metric_out: str,
    opt_roi_roi_metric: InputPathType | None = None,
    opt_dump_text_text_out: str | None = None,
    runner: Runner | None = None,
) -> MetricFalseCorrelationOutputs:
    """
    Compare correlation locally and across/through sulci/gyri.
    
    For each vertex, compute the average correlation within a range of geodesic
    distances that don't cross a sulcus/gyrus, and the correlation to the
    closest vertex crossing a sulcus/gyrus. A vertex is considered to cross a
    sulcus/gyrus if the 3D distance is less than a third of the geodesic
    distance. The output file contains the ratio between these correlations, and
    some additional maps to help explain the ratio.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        surface: the surface to compute geodesic and 3D distance with.
        metric_in: the metric to correlate.
        v_3d_dist: maximum 3D distance to check around each vertex.
        geo_outer: maximum geodesic distance to use for neighboring correlation.
        geo_inner: minimum geodesic distance to use for neighboring correlation.
        metric_out: the output metric.
        opt_roi_roi_metric: select a region of interest that has data: the\
            region, as a metric file.
        opt_dump_text_text_out: dump the raw measures used to a text file: the\
            output text file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricFalseCorrelationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_FALSE_CORRELATION_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-false-correlation")
    cargs.append(execution.input_file(surface))
    cargs.append(execution.input_file(metric_in))
    cargs.append(str(v_3d_dist))
    cargs.append(str(geo_outer))
    cargs.append(str(geo_inner))
    cargs.append(metric_out)
    if opt_roi_roi_metric is not None:
        cargs.extend([
            "-roi",
            execution.input_file(opt_roi_roi_metric)
        ])
    if opt_dump_text_text_out is not None:
        cargs.extend([
            "-dump-text",
            opt_dump_text_text_out
        ])
    ret = MetricFalseCorrelationOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(metric_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "METRIC_FALSE_CORRELATION_METADATA",
    "MetricFalseCorrelationOutputs",
    "metric_false_correlation",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

LABELCONVERT_METADATA = Metadata(
    id="70b1b32bfcb741c07b8d256b92dbe12ed0418e5f.boutiques",
    name="labelconvert",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class LabelconvertConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class LabelconvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `labelconvert(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    image_out: OutputPathType
    """the output image"""


def labelconvert(
    path_in: InputPathType,
    lut_in: InputPathType,
    lut_out: InputPathType,
    image_out: str,
    spine: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[LabelconvertConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> LabelconvertOutputs:
    """
    Convert a connectome node image from one lookup table to another.
    
    Typical usage is to convert a parcellation image provided by some other
    software, based on the lookup table provided by that software, to conform to
    a new lookup table, particularly one where the node indices increment from
    1, in preparation for connectome construction; examples of such target
    lookup table files are provided in share//mrtrix3//labelconvert//, but can
    be created by the user to provide the desired node set // ordering //
    colours.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        path_in: the input image.
        lut_in: the connectome lookup table corresponding to the input image.
        lut_out: the target connectome lookup table for the output image.
        image_out: the output image.
        spine: provide a manually-defined segmentation of the base of the spine\
            where the streamlines terminate, so that this can become a node in the\
            connection matrix.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelconvertOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABELCONVERT_METADATA)
    cargs = []
    cargs.append("labelconvert")
    if spine is not None:
        cargs.extend([
            "-spine",
            execution.input_file(spine)
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(path_in))
    cargs.append(execution.input_file(lut_in))
    cargs.append(execution.input_file(lut_out))
    cargs.append(image_out)
    ret = LabelconvertOutputs(
        root=execution.output_file("."),
        image_out=execution.output_file(image_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "LABELCONVERT_METADATA",
    "LabelconvertConfig",
    "LabelconvertOutputs",
    "labelconvert",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

HISTO_SYNTHESIZE_METADATA = Metadata(
    id="ebfa527b23cadea769992b1ad756ebe6c1f58512.boutiques",
    name="histo_synthesize",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class HistoSynthesizeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `histo_synthesize(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_synthetic_histo: OutputPathType
    """Output file of the synthetic histo volume."""


def histo_synthesize(
    mri_volume: InputPathType,
    histo_volume: InputPathType,
    synthetic_histo: str,
    runner: Runner | None = None,
) -> HistoSynthesizeOutputs:
    """
    Tool for synthesizing histology-like volumes from MRI data.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        mri_volume: Input MRI volume file.
        histo_volume: Input HISTO volume file.
        synthetic_histo: Output synthetic histo volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `HistoSynthesizeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(HISTO_SYNTHESIZE_METADATA)
    cargs = []
    cargs.append("histo_synthesize")
    cargs.append("[OPTIONS]")
    cargs.append(execution.input_file(mri_volume))
    cargs.append(execution.input_file(histo_volume))
    cargs.append(synthetic_histo)
    ret = HistoSynthesizeOutputs(
        root=execution.output_file("."),
        output_synthetic_histo=execution.output_file(synthetic_histo),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "HISTO_SYNTHESIZE_METADATA",
    "HistoSynthesizeOutputs",
    "histo_synthesize",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__AFNI_REFACER_RUN_METADATA = Metadata(
    id="72757fe4907642f44cb063cfa3ee5dc5dda0045b.boutiques",
    name="@afni_refacer_run",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VAfniRefacerRunOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__afni_refacer_run(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_deface: OutputPathType
    """Defaced volume (face+ears replaced with zeros)"""
    output_reface: OutputPathType
    """Refaced volume (face+ears replaced with artificial values)"""
    output_reface_plus: OutputPathType
    """Reface_plused volume (face+ears+skull replaced with artificial values)"""
    output_face: OutputPathType
    """Face+ears used to replace or remove subject data"""
    output_face_plus: OutputPathType
    """Face+ears+skull used to replace subject data"""


def v__afni_refacer_run(
    input_file: InputPathType,
    prefix: str,
    mode_all: bool = False,
    anonymize_output: bool = False,
    cost_function: str | None = None,
    shell_option: str | None = None,
    no_clean: bool = False,
    no_images: bool = False,
    overwrite: bool = False,
    verbose: bool = False,
    runner: Runner | None = None,
) -> VAfniRefacerRunOutputs:
    """
    This script re-faces one input dataset, using a master shell dataset to write
    over the subject's 'face' region.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_file: Name of input dataset; can contain path information.
        prefix: Name of output dataset.
        mode_all: Output three volumes: one defaced, one refaced and one\
            reface_plused.
        anonymize_output: Use 3drefit and nifti_tool to anonymize the output\
            datasets.
        cost_function: Specify any cost function that is allowed by 3dAllineate\
            (default: lpa).
        shell_option: Specify which shell to use. Options:\
            afni_refacer_shell_sym_1.0.nii.gz (traditional),\
            afni_refacer_shell_sym_2.0.nii.gz (more face/neck removal). Default:\
            afni_refacer_shell_sym_1.0.nii.gz.
        no_clean: Don't delete temp working directory (default: remove working\
            directory).
        no_images: Don't make pretty images to automatically view the results\
            of re/defacing.
        overwrite: Final two file outputs will overwrite any existing files of\
            the same name (default: don't do this). NB: this option does not apply\
            to the working directory.
        verbose: Run the 3dAllineate part herein with '-verb' (for verbosity).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VAfniRefacerRunOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__AFNI_REFACER_RUN_METADATA)
    cargs = []
    cargs.append("@afni_refacer_run")
    cargs.extend([
        "-input",
        execution.input_file(input_file)
    ])
    if mode_all:
        cargs.append("-mode_all")
    cargs.extend([
        "-prefix",
        prefix
    ])
    if anonymize_output:
        cargs.append("-anonymize_output")
    if cost_function is not None:
        cargs.extend([
            "-cost",
            cost_function
        ])
    if shell_option is not None:
        cargs.extend([
            "-shell",
            shell_option
        ])
    if no_clean:
        cargs.append("-no_clean")
    if no_images:
        cargs.append("-no_images")
    if overwrite:
        cargs.append("-overwrite")
    if verbose:
        cargs.append("-verb_allin")
    ret = VAfniRefacerRunOutputs(
        root=execution.output_file("."),
        output_deface=execution.output_file(prefix + ".deface.nii.gz"),
        output_reface=execution.output_file(prefix + ".reface.nii.gz"),
        output_reface_plus=execution.output_file(prefix + ".reface_plus.nii.gz"),
        output_face=execution.output_file(prefix + ".face.nii.gz"),
        output_face_plus=execution.output_file(prefix + ".face_plus.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VAfniRefacerRunOutputs",
    "V__AFNI_REFACER_RUN_METADATA",
    "v__afni_refacer_run",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_FFT_METADATA = Metadata(
    id="346ceaa099cde8147ff4ea8529349b8671ac8691.boutiques",
    name="3dFFT",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dFftOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_fft(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dataset: OutputPathType | None
    """Output dataset generated by 3dFFT."""


def v_3d_fft(
    dataset: InputPathType,
    abs_: bool = False,
    phase: bool = False,
    complex_: bool = False,
    inverse: bool = False,
    lx: float | None = None,
    ly: float | None = None,
    lz: float | None = None,
    alt_in: bool = False,
    alt_out: bool = False,
    input_: InputPathType | None = None,
    prefix: str | None = None,
    runner: Runner | None = None,
) -> V3dFftOutputs:
    """
    Performs the FFT of the input dataset in 3 directions (x, y, z) and produces the
    output dataset.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        dataset: Input dataset (e.g., dataset.nii).
        abs_: Outputs the magnitude of the FFT (default).
        phase: Outputs the phase of the FFT (-PI..PI).
        complex_: Outputs the complex-valued FFT.
        inverse: Does the inverse FFT instead of the forward FFT.
        lx: Use FFT of length 'xx' in the x-direction.
        ly: Use FFT of length 'yy' in the y-direction.
        lz: Use FFT of length 'zz' in the z-direction.
        alt_in: Alternate signs of input data before FFT to bring zero\
            frequency from edge of FFT-space to center of grid for cosmetic\
            purposes.
        alt_out: Alternate signs of output data after FFT. Use '-altOUT' with\
            '-altIN' on the forward transform to get the signs of the recovered\
            image correct.
        input_: Read the input dataset from specified file instead of from the\
            last argument on the command line.
        prefix: Use specified prefix for the output dataset.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dFftOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_FFT_METADATA)
    cargs = []
    cargs.append("3dFFT")
    cargs.append(execution.input_file(dataset))
    if abs_:
        cargs.append("--abs")
    if phase:
        cargs.append("--phase")
    if complex_:
        cargs.append("--complex")
    if inverse:
        cargs.append("--inverse")
    if lx is not None:
        cargs.extend([
            "--Lx",
            str(lx)
        ])
    if ly is not None:
        cargs.extend([
            "--Ly",
            str(ly)
        ])
    if lz is not None:
        cargs.extend([
            "--Lz",
            str(lz)
        ])
    if alt_in:
        cargs.append("--altIN")
    if alt_out:
        cargs.append("--altOUT")
    if input_ is not None:
        cargs.extend([
            "--input",
            execution.input_file(input_)
        ])
    if prefix is not None:
        cargs.extend([
            "--prefix",
            prefix
        ])
    ret = V3dFftOutputs(
        root=execution.output_file("."),
        output_dataset=execution.output_file(prefix + ".nii.gz") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dFftOutputs",
    "V_3D_FFT_METADATA",
    "v_3d_fft",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

RUN_QDEC_GLM_METADATA = Metadata(
    id="b83d7c50b5137b0f5530f2c1776edefd9de98fa2.boutiques",
    name="run-qdec-glm",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class RunQdecGlmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `run_qdec_glm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def run_qdec_glm(
    qdec_directory: str,
    runner: Runner | None = None,
) -> RunQdecGlmOutputs:
    """
    QDEC GLM (General Linear Model) execution tool for statistical analysis.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        qdec_directory: Directory containing QDEC data.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RunQdecGlmOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(RUN_QDEC_GLM_METADATA)
    cargs = []
    cargs.extend([
        "-glm",
        "run-qdec" + qdec_directory
    ])
    ret = RunQdecGlmOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "RUN_QDEC_GLM_METADATA",
    "RunQdecGlmOutputs",
    "run_qdec_glm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_TCORRELATE_METADATA = Metadata(
    id="a874b3865df8702baf89b243d1e11b051abda5bf.boutiques",
    name="3dTcorrelate",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dTcorrelateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_tcorrelate(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_file: OutputPathType | None
    """Output image file name."""


def v_3d_tcorrelate(
    xset: InputPathType,
    yset: InputPathType,
    pearson: bool = False,
    spearman: bool = False,
    quadrant: bool = False,
    ktaub: bool = False,
    covariance: bool = False,
    partial: InputPathType | None = None,
    ycoef: bool = False,
    fisher: bool = False,
    polort: int | None = None,
    ort: InputPathType | None = None,
    autoclip: bool = False,
    automask: bool = False,
    zcensor: bool = False,
    prefix: str | None = None,
    runner: Runner | None = None,
) -> V3dTcorrelateOutputs:
    """
    3dTcorrelate. Computes the correlation coefficient between corresponding voxel
    time series in two input 3D+time datasets 'xset' and 'yset'.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        xset: Input xset.
        yset: Input yset.
        pearson: Correlation is the normal pearson correlation coefficient.
        spearman: Correlation is the Spearman (rank) correlation coefficient.
        quadrant: Correlation is the quadrant coefficient.
        ktaub: Correlation is Kendall's tau_b coefficient. For continuous or\
            finely discretized data, tau_b and rank correlation are nearly\
            equivalent.
        covariance: Covariance instead of correlation. That would be Pearson\
            correlation without scaling by the product of the standard deviations.
        partial: Partial Pearson's correlation of X & Y, adjusting for Z (the\
            dataset provided here).
        ycoef: Least squares coefficient that best fits y(t) to x(t), after\
            detrending. That is, if yd(t) is the detrended y(t) and xd(t) is the\
            detrended x(t), then the ycoef value is from the OLSQ fit to xd(t) =\
            ycoef & y(t) + error.
        fisher: Apply the Fisher (inverse hyperbolic tangent) transformation to\
            correlation results. Does not make sense with ktaub, covariance, or\
            ycoef.
        polort: Remove polynomial trend of order m. Using m=-1 mean no\
            detrending; this is only useful fro data that has been preprocessed.
        ort: A 1D file. Also detrend using the columbs of the 1D file provided\
            here. Only one -ort option can be given, so if you would like to use\
            more than one, create a temporary file using 1dcat.
        autoclip: Clip off low-intensity regions in the two datasets, so that\
            the correlation is only computed between high-intensity (presumably\
            brain) voxels. The intensity level is determined the same way that\
            3dClipLevel works.
        automask: Clip off low-intensity regions in the two datasets, so that\
            the correlation is only computed between high-intensity (presumably\
            brain) voxels. The intensity level is determined the same way that\
            3dClipLevel works.
        zcensor: Omit (censor out) any time points where the xset volume is all\
            zero OR where the yset volume is all zero (in mask). Please note that\
            using -zcensor with any detrending is unlikely to be useful.
        prefix: Save output into a dataset with this prefix.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dTcorrelateOutputs`).
    """
    if polort is not None and not (-1 <= polort <= 9): 
        raise ValueError(f"'polort' must be between -1 <= x <= 9 but was {polort}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_TCORRELATE_METADATA)
    cargs = []
    cargs.append("3dTcorrelate")
    cargs.append(execution.input_file(xset))
    cargs.append(execution.input_file(yset))
    if pearson:
        cargs.append("-pearson")
    if spearman:
        cargs.append("-spearman")
    if quadrant:
        cargs.append("-quadrant")
    if ktaub:
        cargs.append("-ktaub")
    if covariance:
        cargs.append("-covariance")
    if partial is not None:
        cargs.extend([
            "-partial",
            execution.input_file(partial)
        ])
    if ycoef:
        cargs.append("-ycoef")
    if fisher:
        cargs.append("-Fisher")
    if polort is not None:
        cargs.extend([
            "-polort",
            str(polort)
        ])
    if ort is not None:
        cargs.extend([
            "-ort",
            execution.input_file(ort)
        ])
    if autoclip:
        cargs.append("-autoclip")
    if automask:
        cargs.append("-automask")
    if zcensor:
        cargs.append("-zcensor")
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    ret = V3dTcorrelateOutputs(
        root=execution.output_file("."),
        out_file=execution.output_file(prefix) if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dTcorrelateOutputs",
    "V_3D_TCORRELATE_METADATA",
    "v_3d_tcorrelate",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_SURFACE_STATS_METADATA = Metadata(
    id="a615c38b1e647ea0789fa48d75a57ab954bd3b51.boutiques",
    name="mris_surface_stats",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisSurfaceStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_surface_stats(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    std_output: OutputPathType
    """Standard deviation map of the input thickness difference maps"""
    mean_output: OutputPathType | None
    """Mean map of the input thickness difference maps"""
    absmean_output: OutputPathType | None
    """Absolute mean map of the input thickness difference maps"""
    absstd_output: OutputPathType | None
    """Standard deviation map of the absolute differences"""
    zscore_output: OutputPathType | None
    """Z-score map of the input thickness difference maps"""


def mris_surface_stats(
    surf_name: InputPathType,
    out_name: str,
    data_files: list[InputPathType],
    nsmooth: float | None = None,
    mask_name: InputPathType | None = None,
    mean: str | None = None,
    absmean: str | None = None,
    absstd: str | None = None,
    zscore: str | None = None,
    first_group_size: float | None = None,
    src_type: str | None = None,
    trg_type: str | None = None,
    debug: float | None = None,
    runner: Runner | None = None,
) -> MrisSurfaceStatsOutputs:
    """
    Computes the group-wise mean and standard deviation of thickness differences at
    every vertex of the template surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        surf_name: Set the surface filename.
        out_name: Set the output filename (standard deviation of data).
        data_files: List of input data files for computation.
        nsmooth: Specify number of smoothing steps.
        mask_name: Set the filename for surface mask.
        mean: Set the output filename for mean.
        absmean: Set the output filename for absolute mean.
        absstd: Set the output filename for standard deviation of absolute mean.
        zscore: Set the output filename for z-score (only if first_group_size >\
            0).
        first_group_size: Specify how many subjects at the beginning belong to\
            first group.
        src_type: Input surface data format (default = paint).
        trg_type: Output format (default = paint).
        debug: Specify which surface vertex number to debug.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisSurfaceStatsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_SURFACE_STATS_METADATA)
    cargs = []
    cargs.append("mris_surface_stats")
    if nsmooth is not None:
        cargs.extend([
            "-nsmooth",
            str(nsmooth)
        ])
    cargs.extend([
        "-surf_name",
        execution.input_file(surf_name)
    ])
    if mask_name is not None:
        cargs.extend([
            "-mask_name",
            execution.input_file(mask_name)
        ])
    cargs.extend([
        "-out_name",
        out_name
    ])
    if mean is not None:
        cargs.extend([
            "-mean",
            mean
        ])
    if absmean is not None:
        cargs.extend([
            "-absmean",
            absmean
        ])
    if absstd is not None:
        cargs.extend([
            "-absstd",
            absstd
        ])
    if zscore is not None:
        cargs.extend([
            "-zscore",
            zscore
        ])
    if first_group_size is not None:
        cargs.extend([
            "-first_group_size",
            str(first_group_size)
        ])
    if src_type is not None:
        cargs.extend([
            "-src_type",
            src_type
        ])
    if trg_type is not None:
        cargs.extend([
            "-trg_type",
            trg_type
        ])
    if debug is not None:
        cargs.extend([
            "-debug",
            str(debug)
        ])
    cargs.extend([execution.input_file(f) for f in data_files])
    ret = MrisSurfaceStatsOutputs(
        root=execution.output_file("."),
        std_output=execution.output_file(out_name),
        mean_output=execution.output_file(mean) if (mean is not None) else None,
        absmean_output=execution.output_file(absmean) if (absmean is not None) else None,
        absstd_output=execution.output_file(absstd) if (absstd is not None) else None,
        zscore_output=execution.output_file(zscore) if (zscore is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_SURFACE_STATS_METADATA",
    "MrisSurfaceStatsOutputs",
    "mris_surface_stats",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_SIGNATURES_METADATA = Metadata(
    id="6f7958fe91c1d63deab2f5ca691371fc161ee645.boutiques",
    name="3dSignatures",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dSignaturesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_signatures(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    results_file: OutputPathType
    """Main analysis results file"""


def v_3d_signatures(
    infile: InputPathType,
    outfile: str,
    segmentation: bool = False,
    filter_: bool = False,
    threshold: float | None = None,
    smoothing: float | None = None,
    runner: Runner | None = None,
) -> V3dSignaturesOutputs:
    """
    3dSignatures analysis tool for 3D genome organization.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        infile: Input file containing 3D genome data (e.g. genome_data.txt).
        outfile: Output file to store analysis results (e.g.\
            analysis_results.txt).
        segmentation: Flag to apply genome segmentation.
        filter_: Flag to apply data filtering.
        threshold: Threshold level for data filtering; default=0.5.
        smoothing: Apply smoothing with specified kernel size.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dSignaturesOutputs`).
    """
    if threshold is not None and not (0 <= threshold <= 1): 
        raise ValueError(f"'threshold' must be between 0 <= x <= 1 but was {threshold}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_SIGNATURES_METADATA)
    cargs = []
    cargs.append("3dSignatures")
    cargs.append(execution.input_file(infile))
    cargs.append(outfile)
    if segmentation:
        cargs.append("--segmentation")
    if filter_:
        cargs.append("--filter")
    if threshold is not None:
        cargs.extend([
            "--threshold",
            str(threshold)
        ])
    if smoothing is not None:
        cargs.extend([
            "--smoothing",
            str(smoothing)
        ])
    ret = V3dSignaturesOutputs(
        root=execution.output_file("."),
        results_file=execution.output_file(outfile + ".txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dSignaturesOutputs",
    "V_3D_SIGNATURES_METADATA",
    "v_3d_signatures",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_JACOBIAN_METADATA = Metadata(
    id="f32baf37cb24be970160e497e65bff0c90b9d331.boutiques",
    name="mris_jacobian",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisJacobianOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_jacobian(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_jacobian_file: OutputPathType
    """Output file containing the Jacobian of the surface mapping."""


def mris_jacobian(
    original_surface: InputPathType,
    mapped_surface: InputPathType,
    jacobian_file: str,
    log: bool = False,
    noscale: bool = False,
    invert: bool = False,
    runner: Runner | None = None,
) -> MrisJacobianOutputs:
    """
    This program computes the Jacobian of a surface mapping.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        original_surface: The original surface file.
        mapped_surface: The mapped surface file.
        jacobian_file: The output file name for the Jacobian.
        log: Compute and write out log of Jacobian.
        noscale: Don't scale Jacobian by total surface areas.
        invert: Compute -1/Jacobian for Jacobian < 1.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisJacobianOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_JACOBIAN_METADATA)
    cargs = []
    cargs.append("mris_jacobian")
    cargs.append(execution.input_file(original_surface))
    cargs.append(execution.input_file(mapped_surface))
    cargs.append(jacobian_file)
    if log:
        cargs.append("-log")
    if noscale:
        cargs.append("-noscale")
    if invert:
        cargs.append("-invert")
    ret = MrisJacobianOutputs(
        root=execution.output_file("."),
        output_jacobian_file=execution.output_file(jacobian_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_JACOBIAN_METADATA",
    "MrisJacobianOutputs",
    "mris_jacobian",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_COMPUTE_VOLUME_INTENSITIES_METADATA = Metadata(
    id="0ef3728f055c825945a1bf63696573fc7f07f67d.boutiques",
    name="mri_compute_volume_intensities",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriComputeVolumeIntensitiesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_compute_volume_intensities(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_volume_file: OutputPathType
    """Computed output volume file"""


def mri_compute_volume_intensities(
    input_intensity: InputPathType,
    volume_fraction_stem: str,
    output_volume: InputPathType,
    runner: Runner | None = None,
) -> MriComputeVolumeIntensitiesOutputs:
    """
    A tool to compute volume intensities for a given input intensity volume and
    volume fraction stem.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_intensity: Input intensity volume.
        volume_fraction_stem: Volume fraction stem.
        output_volume: Output volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriComputeVolumeIntensitiesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_COMPUTE_VOLUME_INTENSITIES_METADATA)
    cargs = []
    cargs.append("mri_compute_volume_intensities")
    cargs.append(execution.input_file(input_intensity))
    cargs.append(volume_fraction_stem)
    cargs.append(execution.input_file(output_volume))
    ret = MriComputeVolumeIntensitiesOutputs(
        root=execution.output_file("."),
        output_volume_file=execution.output_file(pathlib.Path(output_volume).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_COMPUTE_VOLUME_INTENSITIES_METADATA",
    "MriComputeVolumeIntensitiesOutputs",
    "mri_compute_volume_intensities",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_LINEAR_ALIGN_BINARY_METADATA = Metadata(
    id="8f952429c373760f2ebf14b9f290bc6f0c53127a.boutiques",
    name="mri_linear_align_binary",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriLinearAlignBinaryOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_linear_align_binary(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_xform_file: OutputPathType
    """The output transformation file"""


def mri_linear_align_binary(
    source: InputPathType,
    target: InputPathType,
    output_xform: str,
    target_label: str | None = None,
    runner: Runner | None = None,
) -> MriLinearAlignBinaryOutputs:
    """
    A tool for linear alignment of MRI images.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        source: Source image file.
        target: Target image file.
        output_xform: Output transformation file.
        target_label: Target label, assume source is high-resolution\
            hippocampal labeling.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriLinearAlignBinaryOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_LINEAR_ALIGN_BINARY_METADATA)
    cargs = []
    cargs.append("mri_linear_align_binary")
    cargs.append(execution.input_file(source))
    cargs.append(execution.input_file(target))
    cargs.append(output_xform)
    if target_label is not None:
        cargs.extend([
            "-h",
            target_label
        ])
    ret = MriLinearAlignBinaryOutputs(
        root=execution.output_file("."),
        output_xform_file=execution.output_file(output_xform),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_LINEAR_ALIGN_BINARY_METADATA",
    "MriLinearAlignBinaryOutputs",
    "mri_linear_align_binary",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FABBER_DUALECHO_METADATA = Metadata(
    id="3b814c58103c730b38e41b1c598b309b06d4ea6b.boutiques",
    name="fabber_dualecho",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class FabberDualechoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fabber_dualecho(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_files: OutputPathType
    """All output files will be stored in the output directory specified by the
    user."""


def fabber_dualecho(
    output_directory: str,
    method: str,
    model: str,
    data: InputPathType,
    data_order: str | None = "interleave",
    mask_file: InputPathType | None = None,
    mt_list: float | None = None,
    supp_data: InputPathType | None = None,
    options_file: InputPathType | None = None,
    help_flag: bool = False,
    list_methods_flag: bool = False,
    list_models_flag: bool = False,
    list_params_flag: bool = False,
    desc_params_flag: bool = False,
    list_outputs_flag: bool = False,
    evaluate: str | None = None,
    evaluate_params: str | None = None,
    evaluate_nt: float | None = None,
    simple_output_flag: bool = False,
    overwrite_flag: bool = False,
    link_to_latest_flag: bool = False,
    load_models: InputPathType | None = None,
    dump_param_names_flag: bool = False,
    save_model_fit_flag: bool = False,
    save_residuals_flag: bool = False,
    save_model_extras_flag: bool = False,
    save_mvn_flag: bool = False,
    save_mean_flag: bool = False,
    save_std_flag: bool = False,
    save_var_flag: bool = False,
    save_zstat_flag: bool = False,
    save_noise_mean_flag: bool = False,
    save_noise_std_flag: bool = False,
    save_free_energy_flag: bool = False,
    debug_flag: bool = False,
    runner: Runner | None = None,
) -> FabberDualechoOutputs:
    """
    FMRIB's Advanced Bayesian Estimation and Inference tool for FMRI analysis.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        output_directory: Directory for output files (including logfile).
        method: Use this inference method.
        model: Use this forward model.
        data: Specify a single input data file.
        data_order: If multiple data files are specified, how they will be\
            handled: concatenate = one after the other, interleave = first record\
            from each file, then second, etc.
        mask_file: Mask file. Inference will only be performed where mask value\
            > 0.
        mt_list: List of masked time points, indexed from 1. These will be\
            ignored in the parameter updates.
        supp_data: 'Supplemental' timeseries data, required for some models.
        options_file: File containing additional options, one per line, in the\
            same form as specified on the command line.
        help_flag: Print this usage method. If given with --method or --model,\
            display relevant method/model usage information.
        list_methods_flag: List all known inference methods.
        list_models_flag: List all known forward models.
        list_params_flag: List model parameters (requires model configuration\
            options to be given).
        desc_params_flag: Descript model parameters (name, description, units)\
            - requires model configuration options to be given. Note that not all\
            models provide parameter descriptions.
        list_outputs_flag: List additional model outputs (requires model\
            configuration options to be given).
        evaluate: Evaluate model. Set to name of output required or blank for\
            default output. Requires model configuration options, --evaluate-params\
            and --evaluate-nt.
        evaluate_params: List of parameter values for evaluation.
        evaluate_nt: Number of time points for evaluation - must be consistent\
            with model options where appropriate.
        simple_output_flag: Instead of usual standard output, simply output\
            series of lines each giving progress as percentage.
        overwrite_flag: If set will overwrite existing output. If not set, new\
            output directories will be created by appending '+' to the directory\
            name.
        link_to_latest_flag: Try to create a link to the most recent output\
            directory with the prefix _latest.
        load_models: Load models dynamically from the specified filename, which\
            should be a DLL/shared library.
        dump_param_names_flag: Write the file paramnames.txt containing the\
            names of the model parameters.
        save_model_fit_flag: Output the model prediction as a 4d volume.
        save_residuals_flag: Output the residuals (difference between the data\
            and the model prediction).
        save_model_extras_flag: Output any additional model-specific timeseries\
            data.
        save_mvn_flag: Output the final MVN distributions.
        save_mean_flag: Output the parameter means.
        save_std_flag: Output the parameter standard deviations.
        save_var_flag: Output the parameter variances.
        save_zstat_flag: Output the parameter Zstats.
        save_noise_mean_flag: Output the noise means. The noise distribution\
            inferred is the precision of a Gaussian noise source.
        save_noise_std_flag: Output the noise standard deviations.
        save_free_energy_flag: Output the free energy, if calculated.
        debug_flag: Output large amounts of debug information. ONLY USE WITH\
            VERY SMALL NUMBERS OF VOXELS.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FabberDualechoOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FABBER_DUALECHO_METADATA)
    cargs = []
    cargs.append("fabber_dualecho")
    cargs.append(output_directory)
    cargs.extend([
        "--method",
        method
    ])
    cargs.extend([
        "--model",
        model
    ])
    cargs.extend([
        "--data",
        execution.input_file(data)
    ])
    if data_order is not None:
        cargs.extend([
            "--data-order",
            data_order
        ])
    if mask_file is not None:
        cargs.extend([
            "--mask",
            execution.input_file(mask_file)
        ])
    if mt_list is not None:
        cargs.extend([
            "--mt<n>",
            str(mt_list)
        ])
    if supp_data is not None:
        cargs.extend([
            "--suppdata",
            execution.input_file(supp_data)
        ])
    if options_file is not None:
        cargs.extend([
            "--optfile",
            execution.input_file(options_file)
        ])
    if help_flag:
        cargs.append("--help")
    if list_methods_flag:
        cargs.append("--listmethods")
    if list_models_flag:
        cargs.append("--listmodels")
    if list_params_flag:
        cargs.append("--listparams")
    if desc_params_flag:
        cargs.append("--descparams")
    if list_outputs_flag:
        cargs.append("--listoutputs")
    if evaluate is not None:
        cargs.extend([
            "--evaluate",
            evaluate
        ])
    if evaluate_params is not None:
        cargs.extend([
            "--evaluate-params",
            evaluate_params
        ])
    if evaluate_nt is not None:
        cargs.extend([
            "--evaluate-nt",
            str(evaluate_nt)
        ])
    if simple_output_flag:
        cargs.append("--simple-output")
    if overwrite_flag:
        cargs.append("--overwrite")
    if link_to_latest_flag:
        cargs.append("--link-to-latest")
    if load_models is not None:
        cargs.extend([
            "--loadmodels",
            execution.input_file(load_models)
        ])
    if dump_param_names_flag:
        cargs.append("--dump-param-names")
    if save_model_fit_flag:
        cargs.append("--save-model-fit")
    if save_residuals_flag:
        cargs.append("--save-residuals")
    if save_model_extras_flag:
        cargs.append("--save-model-extras")
    if save_mvn_flag:
        cargs.append("--save-mvn")
    if save_mean_flag:
        cargs.append("--save-mean")
    if save_std_flag:
        cargs.append("--save-std")
    if save_var_flag:
        cargs.append("--save-var")
    if save_zstat_flag:
        cargs.append("--save-zstat")
    if save_noise_mean_flag:
        cargs.append("--save-noise-mean")
    if save_noise_std_flag:
        cargs.append("--save-noise-std")
    if save_free_energy_flag:
        cargs.append("--save-free-energy")
    if debug_flag:
        cargs.append("--debug")
    ret = FabberDualechoOutputs(
        root=execution.output_file("."),
        output_files=execution.output_file(output_directory + "/*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FABBER_DUALECHO_METADATA",
    "FabberDualechoOutputs",
    "fabber_dualecho",
]
